@@ -123,10 +123,8 @@ __glXInitVertexArrayState( __GLXcontext * gc )
     struct array_state_vector * arrays;
 
     unsigned array_count;
-    unsigned texture_units = 1;
-    unsigned i;
-    unsigned j;
-    unsigned vertex_program_attribs = 0;
+    int texture_units = 1, vertex_program_attribs = 0;
+    unsigned i, j;
 
     GLboolean got_fog = GL_FALSE;
     GLboolean got_secondary_color = GL_FALSE;
@@ -1591,7 +1589,7 @@ void __indirect_glClientActiveTextureARB(GLenum texture)
     const GLint unit = (GLint) texture - GL_TEXTURE0;
 
 
-    if ( (unit < 0) || (unit > arrays->num_texture_units) ) {
+    if ( (unit < 0) || (unit >= arrays->num_texture_units) ) {
 	__glXSetError(gc, GL_INVALID_ENUM);
 	return;
     }
@@ -1766,13 +1764,14 @@ __glXPushArrayState( __GLXattribute * state )
     struct array_stack_state * stack = & arrays->stack[ (arrays->stack_index * arrays->num_arrays)];
     unsigned  i;
 
-
+    /* XXX are we pushing _all_ the necessary fields? */
     for ( i = 0 ; i < arrays->num_arrays ; i++ ) {
 	stack[i].data = arrays->arrays[i].data;
 	stack[i].data_type = arrays->arrays[i].data_type;
 	stack[i].user_stride = arrays->arrays[i].user_stride;
 	stack[i].count = arrays->arrays[i].count;
 	stack[i].key = arrays->arrays[i].key;
+        stack[i].index = arrays->arrays[i].index;
 	stack[i].enabled = arrays->arrays[i].enabled;
     }
 
