@@ -43,6 +43,7 @@
 #include "fbobject.h"
 #include "texrender.h"
 #endif
+#include "arrayobj.h"
 
 #include "driverfuncs.h"
 #include "tnl/tnl.h"
@@ -125,6 +126,9 @@ _mesa_init_driver_functions(struct dd_function_table *driver)
    driver->BindProgram = NULL;
    driver->NewProgram = _mesa_new_program;
    driver->DeleteProgram = _mesa_delete_program;
+#if FEATURE_MESA_program_debug
+   driver->GetFragmentProgramRegister = _swrast_get_program_register;
+#endif /* FEATURE_MESA_program_debug */
 
    /* simple state commands */
    driver->AlphaFunc = NULL;
@@ -220,6 +224,11 @@ _mesa_init_driver_functions(struct dd_function_table *driver)
    driver->BeginQuery = NULL;
    driver->EndQuery = NULL;
 
+   /* APPLE_vertex_array_object */
+   driver->NewArrayObject = _mesa_new_array_object;
+   driver->DeleteArrayObject = _mesa_delete_array_object;
+   driver->BindArrayObject = NULL;
+
    /* T&L stuff */
    driver->NeedValidate = GL_FALSE;
    driver->ValidateTnlModule = NULL;
