@@ -205,8 +205,7 @@ static void gammaDDBlendFuncSeparate( GLcontext *ctx,
  * Buffer clear
  */
 
-static void gammaDDClear( GLcontext *ctx, GLbitfield mask, GLboolean all,
-			       GLint cx, GLint cy, GLint cw, GLint ch )
+static void gammaDDClear( GLcontext *ctx, GLbitfield mask )
 {
    gammaContextPtr gmesa = GAMMA_CONTEXT(ctx);
    GLINTDRIPtr gDRIPriv = (GLINTDRIPtr)gmesa->driScreen->pDevPriv;
@@ -410,7 +409,7 @@ static void gammaDDClear( GLcontext *ctx, GLbitfield mask, GLboolean all,
 #endif
 
    if ( mask )
-      _swrast_Clear( ctx, mask, all, cx, cy, cw, ch );
+      _swrast_Clear( ctx, mask );
 }
 
 /* =============================================================
@@ -1099,8 +1098,6 @@ void gammaUpdateWindow( GLcontext *ctx )
 static void gammaDDViewport( GLcontext *ctx, GLint x, GLint y,
 			    GLsizei width, GLsizei height )
 {
-   /* update size of Mesa/software ancillary buffers */
-   _mesa_ResizeBuffersMESA();
    gammaUpdateWindow( ctx );
 }
 
@@ -1722,9 +1719,5 @@ void gammaDDInitStateFuncs( GLcontext *ctx )
    ctx->Driver.PolygonStipple		= gammaDDPolygonStipple;
    ctx->Driver.Scissor			= gammaDDScissor;
    ctx->Driver.ShadeModel		= gammaDDShadeModel;
-   ctx->Driver.ClearStencil		= NULL;
-   ctx->Driver.StencilFunc		= NULL;
-   ctx->Driver.StencilMask		= NULL;
-   ctx->Driver.StencilOp		= NULL;
    ctx->Driver.Viewport			= gammaDDViewport;
 }
