@@ -47,11 +47,13 @@ WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "colormac.h"
 #include "radeon_context.h"
 
-/* PPC doesnt support 16 bit elts ... */
-#ifndef MESA_BIG_ENDIAN
 #define USER_BUFFERS
 #define RADEON_VTXFMT_A
 #define HW_VBOS
+
+/* We don't handle 16 bits elts swapping yet */
+#ifdef MESA_BIG_ENDIAN
+#define FORCE_32BITS_ELTS
 #endif
 
 //#define OPTIMIZE_ELTS
@@ -129,7 +131,6 @@ struct r300_dma_region {
     int aos_offset;     /* address in GART memory */
     int aos_stride;     /* distance between elements, in dwords */
     int aos_size;       /* number of components (1-4) */
-    int aos_format;     /* format of components */
     int aos_reg;        /* VAP register assignment */
 };
 
@@ -206,6 +207,10 @@ struct r300_texture_env_state {
 	GLenum envMode;
 };
 
+
+/* The blit width for texture uploads
+ */
+#define R300_BLIT_WIDTH_BYTES 1024
 #define R300_MAX_TEXTURE_UNITS 8
 
 struct r300_texture_state {
@@ -346,6 +351,21 @@ struct r300_state_atom {
 #define R300_FPP_PARAM_0	1
 #define R300_FPP_CMDSIZE	(32*4+1)
 
+#define R300_FOGS_CMD_0		0
+#define R300_FOGS_STATE		1
+#define R300_FOGS_CMDSIZE	2
+
+#define R300_FOGC_CMD_0		0
+#define R300_FOGC_R		1
+#define R300_FOGC_G		2
+#define R300_FOGC_B		3
+#define R300_FOGC_CMDSIZE	4
+
+#define R300_FOGP_CMD_0		0
+#define R300_FOGP_SCALE		1
+#define R300_FOGP_START		2
+#define R300_FOGP_CMDSIZE	3
+
 #define R300_AT_CMD_0		0
 #define R300_AT_ALPHA_TEST	1
 #define R300_AT_UNKNOWN		2
@@ -432,6 +452,8 @@ struct r300_hw_state {
 	struct r300_state_atom unk4260; /* (4260) */
 	struct r300_state_atom unk4274; /* (4274) */
 	struct r300_state_atom unk4288; /* (4288) */
+	struct r300_state_atom fogp;	/* fog parameters (4294) */
+	struct r300_state_atom unk429C; /* (429C) */
 	struct r300_state_atom unk42A0;	/* (42A0) */
 	struct r300_state_atom zbs;	/* zbias (42A4) */
 	struct r300_state_atom unk42B4; /* (42B4) */
@@ -446,8 +468,8 @@ struct r300_hw_state {
 	struct r300_state_atom fpt;     /* texi - (4620) */
 	struct r300_state_atom unk46A4;	/* (46A4) */
 	struct r300_state_atom fpi[4];	/* fp instructions (46C0/47C0/48C0/49C0) */
-	struct r300_state_atom unk4BC0;	/* (4BC0) */
-	struct r300_state_atom unk4BC8;	/* (4BC8) */
+	struct r300_state_atom fogs;	/* fog state (4BC0) */
+	struct r300_state_atom fogc;	/* fog color (4BC8) */
 	struct r300_state_atom at;	/* alpha test (4BD4) */
 	struct r300_state_atom unk4BD8;	/* (4BD8) */
 	struct r300_state_atom fpp;     /* 0x4C00 and following */
@@ -518,25 +540,16 @@ struct r300_stencilbuffer_state {
 
 };
 
-struct r300_vap_reg_state {
-	   /* input register assigments */
-	   int i_coords;
-	   int i_normal;
-	   int i_color[2];
-	   int i_fog;
-	   int i_tex[R300_MAX_TEXTURE_UNITS];
-	   int i_index;
-	   int i_pointsize;
-	};
-
 /* Vertex shader state */
 
 /* Perhaps more if we store programs in vmem? */
-#define VSF_MAX_FRAGMENT_LENGTH (256*4)
+/* drm_r300_cmd_header_t->vpu->count is unsigned char */
+#define VSF_MAX_FRAGMENT_LENGTH (255*4)
 	
 /* Can be tested with colormat currently. */
 #define VSF_MAX_FRAGMENT_TEMPS (14)
 
+#define STATE_R300_WINDOW_DIMENSION (STATE_INTERNAL_DRIVER+0)
 
 struct r300_vertex_shader_fragment {
 	int length;
@@ -580,7 +593,8 @@ struct r300_vertex_shader_state {
 	
 extern int hw_tcl_on;
 
-#define CURRENT_VERTEX_SHADER(ctx) (ctx->VertexProgram._Current)
+//#define CURRENT_VERTEX_SHADER(ctx) (ctx->VertexProgram._Current)
+#define CURRENT_VERTEX_SHADER(ctx) (R300_CONTEXT(ctx)->selected_vp)
 
 /* Should but doesnt work */
 //#define CURRENT_VERTEX_SHADER(ctx) (R300_CONTEXT(ctx)->curr_vp)
@@ -595,15 +609,22 @@ extern int hw_tcl_on;
 /* r300_vertex_shader_state and r300_vertex_program should probably be merged together someday.
  * Keeping them them seperate for now should ensure fixed pipeline keeps functioning properly.
  */	
+
+struct r300_vertex_program_key {
+	GLuint InputsRead;
+	GLuint OutputsWritten;
+};
+
 struct r300_vertex_program {
-	struct vertex_program mesa_program; /* Must be first */
+	struct r300_vertex_program *next;
+	struct r300_vertex_program_key key;
 	int translated;
 	
 	struct r300_vertex_shader_fragment program;
-	struct r300_vertex_shader_fragment params;
 	
 	int pos_end;
 	int num_temporaries; /* Number of temp vars used by program */
+	int wpos_idx;
 	int inputs[VERT_ATTRIB_MAX];
 	int outputs[VERT_RESULT_MAX];
 	int native;
@@ -611,6 +632,12 @@ struct r300_vertex_program {
 	int use_ref_count;
 };
 
+struct r300_vertex_program_cont {
+	struct gl_vertex_program mesa_program; /* Must be first */
+	struct r300_vertex_shader_fragment params;
+	struct r300_vertex_program *progs;
+};
+
 #define PFS_MAX_ALU_INST	64
 #define PFS_MAX_TEX_INST	64
 #define PFS_MAX_TEX_INDIRECT 4
@@ -650,7 +677,7 @@ struct r300_pfs_compile_state {
 };
 
 struct r300_fragment_program {
-	struct fragment_program mesa_program;
+	struct gl_fragment_program mesa_program;
 
 	GLcontext *ctx;
 	GLboolean translated;
@@ -703,6 +730,7 @@ struct r300_fragment_program {
 
 #define R300_MAX_AOS_ARRAYS		16
 
+#define AOS_FORMAT_USHORT	0
 #define AOS_FORMAT_FLOAT	1
 #define AOS_FORMAT_UBYTE	2
 #define AOS_FORMAT_FLOAT_COLOR	3
@@ -749,7 +777,7 @@ struct r300_aos_rec {
 struct r300_state {
 	struct r300_depthbuffer_state depth;
 	struct r300_texture_state texture;
-	struct r300_vap_reg_state vap_reg;
+	int sw_tcl_inputs[VERT_ATTRIB_MAX];
 	struct r300_vertex_shader_state vertex_shader;
 	struct r300_pfs_compile_state pfs_compile;
 	struct r300_dma_region aos[R300_MAX_AOS_ARRAYS];
@@ -759,7 +787,7 @@ struct r300_state {
 	GLuint *Elts;
 	struct r300_dma_region elt_dma;
 	
-	GLuint render_inputs; /* actual render inputs that R300 was configured for. 
+	DECLARE_RENDERINPUTS(render_inputs_bitset); /* actual render inputs that R300 was configured for. 
 				 They are the same as tnl->render_inputs for fixed pipeline */	
 	
 	struct {
@@ -783,7 +811,8 @@ struct r300_context {
 	struct r300_hw_state hw;
 	struct r300_cmdbuf cmdbuf;
 	struct r300_state state;
-	struct vertex_program *curr_vp;
+	struct gl_vertex_program *curr_vp;
+	struct r300_vertex_program *selected_vp;
 
 	/* Vertex buffers
 	 */
@@ -810,6 +839,8 @@ struct r300_context {
 #endif
 
 	GLboolean texmicrotile;
+	GLboolean span_dlocking;
+	GLboolean disable_lowimpact_fallback;
 };
 
 struct r300_buffer_object {
@@ -837,9 +868,11 @@ extern GLboolean r300CreateContext(const __GLcontextModes * glVisual,
 				   __DRIcontextPrivate * driContextPriv,
 				   void *sharedContextPrivate);
 
-void r300_translate_vertex_shader(struct r300_vertex_program *vp);
+extern int r300_get_num_verts(r300ContextPtr rmesa, int num_verts, int prim);
+
+extern void r300_select_vertex_shader(r300ContextPtr r300);
 extern void r300InitShaderFuncs(struct dd_function_table *functions);
-extern int r300VertexProgUpdateParams(GLcontext *ctx, struct r300_vertex_program *vp, float *dst);
+extern int r300VertexProgUpdateParams(GLcontext *ctx, struct r300_vertex_program_cont *vp, float *dst);
 extern int r300Fallback(GLcontext *ctx);
 
 extern void radeon_vb_to_rvb(r300ContextPtr rmesa, struct radeon_vertex_buffer *rvb, struct vertex_buffer *vb);
