@@ -28,14 +28,14 @@
 /*
  * Authors:
  *   Ben Skeggs <darktama@iinet.net.au>
+ *   Jerome Glisse <j.glisse@gmail.com>
  */
 
 /*TODO'S
  *
- * - COS/SIN/SCS/LIT instructions
+ * - COS/SIN/SCS instructions
  * - Depth write, WPOS/FOGC inputs
  * - FogOption
- * - Negate on individual components (implement in swizzle code?)
  * - Verify results of opcodes for accuracy, I've only checked them
  *   in specific cases.
  * - and more...
@@ -82,7 +82,8 @@ static const struct {
 	{ "LG2", 1, R300_FPI0_OUTC_REPL_ALPHA, R300_FPI2_OUTA_LG2 },
 	{ "RCP", 1, R300_FPI0_OUTC_REPL_ALPHA, R300_FPI2_OUTA_RCP },
 	{ "RSQ", 1, R300_FPI0_OUTC_REPL_ALPHA, R300_FPI2_OUTA_RSQ },
-	{ "REPL_ALPHA", 1, R300_FPI0_OUTC_REPL_ALPHA, PFS_INVAL }
+	{ "REPL_ALPHA", 1, R300_FPI0_OUTC_REPL_ALPHA, PFS_INVAL },
+	{ "CMPH", 3, R300_FPI0_OUTC_CMPH, PFS_INVAL },
 };
 
 #define MAKE_SWZ3(x, y, z) (MAKE_SWIZZLE4(SWIZZLE_##x, \
@@ -180,7 +181,7 @@ static const pfs_reg_t undef = {
 	valid: GL_FALSE
 };
 
-/* constant zero source */
+/* constant one source */
 static const pfs_reg_t pfs_one = {
 	type: REG_TYPE_CONST,
 	index: 0,
@@ -189,7 +190,16 @@ static const pfs_reg_t pfs_one = {
 	valid: GL_TRUE
 };
 
-/* constant one source */
+/* constant half source */
+static const pfs_reg_t pfs_half = {
+	type: REG_TYPE_CONST,
+	index: 0,
+	v_swz: SWIZZLE_HHH,
+	s_swz: SWIZZLE_HALF,
+	valid: GL_TRUE
+};
+
+/* constant zero source */
 static const pfs_reg_t pfs_zero = {
 	type: REG_TYPE_CONST,
 	index: 0,
@@ -319,7 +329,6 @@ static pfs_reg_t emit_param4fv(struct r300_fragment_program *rp,
 	return r;
 }
 
-#if 0
 static pfs_reg_t emit_const4fv(struct r300_fragment_program *rp, GLfloat *cp)
 { 
 	pfs_reg_t r = undef;
@@ -330,13 +339,11 @@ static pfs_reg_t emit_const4fv(struct r300_fragment_program *rp, GLfloat *cp)
 		ERROR("Out of hw constants!\n");
 		return r;
 	}
-	
-	COPY_4V(rp->constant[r.index], cp);
 
+	COPY_4V(rp->constant[r.index], cp);
 	r.valid = GL_TRUE;
 	return r;
 }
-#endif
 
 static __inline pfs_reg_t negate(pfs_reg_t r)
 {
@@ -496,9 +503,6 @@ static pfs_reg_t t_src(struct r300_fragment_program *rp,
 		       struct prog_src_register fpsrc)
 {
 	pfs_reg_t r = undef;
-#if 0
-	pfs_reg_t n = undef;
-#endif
 
 	switch (fpsrc.File) {
 	case PROGRAM_TEMPORARY:
@@ -529,42 +533,8 @@ static pfs_reg_t t_src(struct r300_fragment_program *rp,
 	}
 
 	/* no point swizzling ONE/ZERO/HALF constants... */
-	if (r.v_swz < SWIZZLE_111 && r.s_swz < SWIZZLE_ZERO)
+	if (r.v_swz < SWIZZLE_111 || r.s_swz < SWIZZLE_ZERO)
 		r = do_swizzle(rp, r, fpsrc.Swizzle, fpsrc.NegateBase);
-#if 0
-	/* WRONG! Need to be able to do individual component negation,
-	 * should probably handle this in the swizzling code unless
-	 * all components are negated, then we can do this natively */
-	if ((fpsrc.NegateBase & 0xf) == 0xf)
-		r.negate = GL_TRUE;
-
-	r.negate_s = (fpsrc.NegateBase >> 3) & 1;
-
-	if ((fpsrc.NegateBase & 0x7) == 0x0) {
-		r.negate_v = 0;
-	} else if ((fpsrc.NegateBase & 0x7) == 0x7) {
-		r.negate_v = 1;
-	} else {
-		if (r.type != REG_TYPE_TEMP) {
-			n = get_temp_reg(rp);
-			emit_arith(rp, PFS_OP_MAD, n, 0x7 ^ fpsrc.NegateBase,
-				   keep(r), pfs_one, pfs_zero, 0);
-			r.negate_v = 1;
-			emit_arith(rp, PFS_OP_MAD, n,
-				   fpsrc.NegateBase & 0x7 | WRITEMASK_W,
-				   r, pfs_one, pfs_zero, 0);
-			r.negate_v = 0;
-			r = n;
-		} else {
-			r.negate_v = 1;
-			emit_arith(rp, PFS_OP_MAD, r,
-				   fpsrc.NegateBase & 0x7 | WRITEMASK_W,
-				   r, pfs_one, pfs_zero, 0);
-			r.negate_v = 0;
-		}
-	}
-#endif
-
 	return r;
 }
 
@@ -573,7 +543,7 @@ static pfs_reg_t t_scalar_src(struct r300_fragment_program *rp,
 {
 	struct prog_src_register src = fpsrc;
 	int sc = GET_SWZ(fpsrc.Swizzle, 0); /* X */
-	
+
 	src.Swizzle = ((sc<<0)|(sc<<3)|(sc<<6)|(sc<<9));
 
 	return t_src(rp, src);
@@ -773,13 +743,15 @@ static void emit_tex(struct r300_fragment_program *rp,
 		cs->dest_in_node = 0;
 	}
 	
-	if (rp->cur_node == 0) rp->first_node_has_tex = 1;
+	if (rp->cur_node == 0)
+		rp->first_node_has_tex = 1;
 
-    rp->tex.inst[rp->tex.length++] = 0
-        | (hwsrc << R300_FPITX_SRC_SHIFT)
-        | (hwdest << R300_FPITX_DST_SHIFT)
-        | (unit << R300_FPITX_IMAGE_SHIFT)
-        | (opcode << R300_FPITX_OPCODE_SHIFT); /* not entirely sure about this */
+	rp->tex.inst[rp->tex.length++] = 0
+		| (hwsrc << R300_FPITX_SRC_SHIFT)
+		| (hwdest << R300_FPITX_DST_SHIFT)
+		| (unit << R300_FPITX_IMAGE_SHIFT)
+		/* not entirely sure about this */
+		| (opcode << R300_FPITX_OPCODE_SHIFT);
 
 	cs->dest_in_node |= (1 << hwdest); 
 	if (coord.type != REG_TYPE_CONST)
@@ -884,7 +856,7 @@ static void emit_arith(struct r300_fragment_program *rp, int op,
 
 	vop = r300_fpop[op].v_op;
 	sop = r300_fpop[op].s_op;
-	argc = r300_fpop[op].argc;	
+	argc = r300_fpop[op].argc;
 
 	if ((mask & WRITEMASK_XYZ) || vop == R300_FPI0_OUTC_DP3)
 		emit_vop = GL_TRUE;
@@ -1018,7 +990,7 @@ static void emit_arith(struct r300_fragment_program *rp, int op,
 #if 0
 static pfs_reg_t get_attrib(struct r300_fragment_program *rp, GLuint attr)
 {
-	struct fragment_program *mp = &rp->mesa_program;
+	struct gl_fragment_program *mp = &rp->mesa_program;
 	pfs_reg_t r = undef;
 
 	if (!(mp->Base.InputsRead & (1<<attr))) {
@@ -1035,11 +1007,13 @@ static pfs_reg_t get_attrib(struct r300_fragment_program *rp, GLuint attr)
 
 static GLboolean parse_program(struct r300_fragment_program *rp)
 {	
-	struct fragment_program *mp = &rp->mesa_program;
+	struct gl_fragment_program *mp = &rp->mesa_program;
 	const struct prog_instruction *inst = mp->Base.Instructions;
 	struct prog_instruction *fpi;
 	pfs_reg_t src[3], dest, temp;
-	int flags, mask;
+	pfs_reg_t cnst;
+	int flags, mask = 0;
+	GLfloat cnstv[4] = {0.0, 0.0, 0.0, 0.0};
 
 	if (!inst || inst[0].Opcode == OPCODE_END) {
 		ERROR("empty program?\n");
@@ -1083,7 +1057,64 @@ static GLboolean parse_program(struct r300_fragment_program *rp)
 				   flags);
 			break;
 		case OPCODE_COS:
-			ERROR("COS not implemented\n");
+			/*
+			 * cos using taylor serie:
+			 * cos(x) = 1 - x^2/2! + x^4/4! - x^6/6!
+			 */
+			temp = get_temp_reg(rp);
+			cnstv[0] = 0.5;
+			cnstv[1] = 0.041666667;
+			cnstv[2] = 0.001388889;
+			cnstv[4] = 0.0;
+			cnst = emit_const4fv(rp, cnstv);
+			src[0] = t_scalar_src(rp, fpi->SrcReg[0]);
+
+			emit_arith(rp, PFS_OP_MAD, temp,
+				   WRITEMASK_XYZ,
+				   src[0],
+				   src[0],
+				   pfs_zero,
+				   flags);
+			emit_arith(rp, PFS_OP_MAD, temp,
+				   WRITEMASK_Y | WRITEMASK_Z,
+				   temp, temp,
+				   pfs_zero,
+				   flags);
+			emit_arith(rp, PFS_OP_MAD, temp,
+				   WRITEMASK_Z,
+				   temp,
+				   swizzle(temp, X, X, X, W),
+				   pfs_zero,
+				   flags);
+			emit_arith(rp, PFS_OP_MAD, temp,
+				   WRITEMASK_XYZ,
+				   temp, cnst,
+				   pfs_zero,
+				   flags);
+			emit_arith(rp, PFS_OP_MAD, temp,
+				   WRITEMASK_X,
+				   pfs_one,
+				   pfs_one,
+				   negate(temp),
+				   flags);
+			emit_arith(rp, PFS_OP_MAD, temp,
+				   WRITEMASK_X,
+				   temp,
+				   pfs_one,
+				   swizzle(temp, Y, Y, Y, W),
+				   flags);
+			emit_arith(rp, PFS_OP_MAD, temp,
+				   WRITEMASK_X,
+				   temp,
+				   pfs_one,
+				   negate(swizzle(temp, Z, Z, Z, W)),
+				   flags);
+			emit_arith(rp, PFS_OP_MAD, dest, mask,
+				   swizzle(temp, X, X, X, X),
+				   pfs_one,
+				   pfs_zero,
+				   flags);
+			free_temp(rp, temp);
 			break;
 		case OPCODE_DP3:
 			src[0] = t_src(rp, fpi->SrcReg[0]);
@@ -1179,7 +1210,66 @@ static GLboolean parse_program(struct r300_fragment_program *rp)
 				   flags);
 			break;
 		case OPCODE_LIT:
-			ERROR("LIT not implemented\n");
+			/* LIT
+			 * if (s.x < 0) t.x = 0; else t.x = s.x;
+			 * if (s.y < 0) t.y = 0; else t.y = s.y;
+			 * if (s.w >  128.0) t.w =  128.0; else t.w = s.w;
+			 * if (s.w < -128.0) t.w = -128.0; else t.w = s.w;
+			 * r.x = 1.0
+			 * if (t.x > 0) r.y = pow(t.y, t.w); else r.y = 0;
+			 * Also r.y = 0 if t.y < 0
+			 * For the t.x > 0 FGLRX use the CMPH opcode which
+			 * change the compare to (t.x + 0.5) > 0.5 we may
+			 * save one instruction by doing CMP -t.x 
+			 */
+			cnstv[0] = cnstv[1] = cnstv[2] = cnstv[4] = 0.50001;
+			src[0] = t_src(rp, fpi->SrcReg[0]);
+			temp = get_temp_reg(rp);
+			cnst = emit_const4fv(rp, cnstv);
+			emit_arith(rp, PFS_OP_CMP, temp,
+				   WRITEMASK_X | WRITEMASK_Y,
+				   src[0], pfs_zero, src[0], flags);
+			emit_arith(rp, PFS_OP_MIN, temp, WRITEMASK_Z,
+				   swizzle(keep(src[0]), W, W, W, W),
+				   cnst, undef, flags);
+			emit_arith(rp, PFS_OP_LG2, temp, WRITEMASK_W,
+				   swizzle(temp, Y, Y, Y, Y),
+				   undef, undef, flags);
+			emit_arith(rp, PFS_OP_MAX, temp, WRITEMASK_Z,
+				   temp, negate(cnst), undef, flags);
+			emit_arith(rp, PFS_OP_MAD, temp, WRITEMASK_W,
+				   temp, swizzle(temp, Z, Z, Z, Z),
+				   pfs_zero, flags);
+			emit_arith(rp, PFS_OP_EX2, temp, WRITEMASK_W,
+				   temp, undef, undef, flags);
+			emit_arith(rp, PFS_OP_MAD, dest, WRITEMASK_Y,
+				   swizzle(keep(temp), X, X, X, X),
+				   pfs_one, pfs_zero, flags);
+#if 0
+			emit_arith(rp, PFS_OP_MAD, temp, WRITEMASK_X,
+				   temp, pfs_one, pfs_half, flags);
+			emit_arith(rp, PFS_OP_CMPH, temp, WRITEMASK_Z,
+				   swizzle(keep(temp), W, W, W, W),
+				   pfs_zero, swizzle(keep(temp), X, X, X, X),
+				   flags);
+#else
+			emit_arith(rp, PFS_OP_CMP, temp, WRITEMASK_Z,
+				   pfs_zero,
+				   swizzle(keep(temp), W, W, W, W),
+				   negate(swizzle(keep(temp), X, X, X, X)),
+				   flags);
+#endif
+			emit_arith(rp, PFS_OP_CMP, dest, WRITEMASK_Z,
+				   pfs_zero, temp,
+				   negate(swizzle(keep(temp), Y, Y, Y, Y)),
+				   flags);
+			emit_arith(rp, PFS_OP_MAD, dest,
+				   WRITEMASK_X | WRITEMASK_W,
+				   pfs_one,
+				   pfs_one,
+				   pfs_zero,
+				   flags);
+			free_temp(rp, temp);
 			break;
 		case OPCODE_LRP:
 			src[0] = t_src(rp, fpi->SrcReg[0]);
@@ -1281,7 +1371,70 @@ static GLboolean parse_program(struct r300_fragment_program *rp)
 			free_temp(rp, temp);
 			break;
 		case OPCODE_SIN:
-			ERROR("SIN not implemented\n");
+			/*
+			 * sin using taylor serie:
+			 * sin(x) = x - x^3/3! + x^5/5! - x^7/7!
+			 */
+			temp = get_temp_reg(rp);
+			cnstv[0] = 0.333333333;
+			cnstv[1] = 0.008333333;
+			cnstv[2] = 0.000198413;
+			cnstv[4] = 0.0;
+			cnst = emit_const4fv(rp, cnstv);
+			src[0] = t_scalar_src(rp, fpi->SrcReg[0]);
+
+			emit_arith(rp, PFS_OP_MAD, temp,
+				   WRITEMASK_XYZ,
+				   src[0],
+				   src[0],
+				   pfs_zero,
+				   flags);
+			emit_arith(rp, PFS_OP_MAD, temp,
+				   WRITEMASK_Y | WRITEMASK_Z,
+				   temp, temp,
+				   pfs_zero,
+				   flags);
+			emit_arith(rp, PFS_OP_MAD, temp,
+				   WRITEMASK_Z,
+				   temp,
+				   swizzle(temp, X, X, X, W),
+				   pfs_zero,
+				   flags);
+			emit_arith(rp, PFS_OP_MAD, temp,
+				   WRITEMASK_XYZ,
+				   src[0],
+				   temp,
+				   pfs_zero,
+				   flags);
+			emit_arith(rp, PFS_OP_MAD, temp,
+				   WRITEMASK_XYZ,
+				   temp, cnst,
+				   pfs_zero,
+				   flags);
+			emit_arith(rp, PFS_OP_MAD, temp,
+				   WRITEMASK_X,
+				   src[0],
+				   pfs_one,
+				   negate(temp),
+				   flags);
+			emit_arith(rp, PFS_OP_MAD, temp,
+				   WRITEMASK_X,
+				   temp,
+				   pfs_one,
+				   swizzle(temp, Y, Y, Y, W),
+				   flags);
+			emit_arith(rp, PFS_OP_MAD, temp,
+				   WRITEMASK_X,
+				   temp,
+				   pfs_one,
+				   negate(swizzle(temp, Z, Z, Z, W)),
+				   flags);
+			emit_arith(rp, PFS_OP_MAD, dest, mask,
+				   swizzle(temp, X, X, X, X),
+				   pfs_one,
+				   pfs_zero,
+				   flags);
+			free_temp(rp, temp);
 			break;
 		case OPCODE_SLT:
 			src[0] = t_src(rp, fpi->SrcReg[0]);
@@ -1345,7 +1498,7 @@ static GLboolean parse_program(struct r300_fragment_program *rp)
 			return GL_FALSE;
 
 	}
-	
+
 	return GL_TRUE;
 }
 
@@ -1355,7 +1508,7 @@ static GLboolean parse_program(struct r300_fragment_program *rp)
 static void init_program(struct r300_fragment_program *rp)
 {
 	struct r300_pfs_compile_state *cs = NULL;
-	struct fragment_program *mp = &rp->mesa_program;	
+	struct gl_fragment_program *mp = &rp->mesa_program;	
 	struct prog_instruction *fpi;
 	GLuint InputsRead = mp->Base.InputsRead;
 	GLuint temps_used = 0; /* for rp->temps[] */
@@ -1400,6 +1553,13 @@ static void init_program(struct r300_fragment_program *rp)
 	}
 	InputsRead &= ~FRAG_BITS_TEX_ANY;
 
+	/* fragment position treated as a texcoord */
+	if (InputsRead & FRAG_BIT_WPOS) {
+		cs->inputs[FRAG_ATTRIB_WPOS].refcount = 0;
+		cs->inputs[FRAG_ATTRIB_WPOS].reg = get_hw_temp(rp);
+	}
+	InputsRead &= ~FRAG_BIT_WPOS;
+
 	/* Then primary colour */
 	if (InputsRead & FRAG_BIT_COL0) {
 		cs->inputs[FRAG_ATTRIB_COL0].refcount = 0;
@@ -1467,7 +1627,7 @@ static void init_program(struct r300_fragment_program *rp)
 
 static void update_params(struct r300_fragment_program *rp)
 {
-	struct fragment_program *mp = &rp->mesa_program;
+	struct gl_fragment_program *mp = &rp->mesa_program;
 	int i;
 
 	/* Ask Mesa nicely to fill in ParameterValues for us */
