@@ -65,12 +65,12 @@ static void r300ClearBuffer(r300ContextPtr r300, int flags, int buffer)
 	__DRIdrawablePrivate *dPriv = r300->radeon.dri.drawable;
 	GLuint cboffset, cbpitch;
 	drm_r300_cmd_header_t* cmd2;
-#ifdef CB_DPATH
+	int cmd_reserved = 0;
+	int cmd_written = 0;
+	drm_radeon_cmd_header_t *cmd = NULL;
 	r300ContextPtr rmesa=r300;
-	LOCAL_VARS;
-#else
-	r300ContextPtr rmesa=r300;
-	LOCAL_VARS;
+
+#ifndef CB_DPATH
 	int i;
 #endif
 	
@@ -218,32 +218,18 @@ static void r300ClearBuffer(r300ContextPtr r300, int flags, int buffer)
 	r300->hw.vpi.cmd[8] = 0;
 
 	R300_STATECHANGE(r300, zs);
+	r300->hw.zs.cmd[R300_ZS_CNTL_0] = 0;
+	r300->hw.zs.cmd[R300_ZS_CNTL_1] = 0;
 	if (flags & CLEARBUFFER_DEPTH) {
-		r300->hw.zs.cmd[R300_ZS_CNTL_0] &= R300_RB3D_STENCIL_ENABLE;
-		r300->hw.zs.cmd[R300_ZS_CNTL_0] |= 0x6; // test and write
-		r300->hw.zs.cmd[R300_ZS_CNTL_1] &= ~(R300_ZS_MASK << R300_RB3D_ZS1_DEPTH_FUNC_SHIFT);
+		r300->hw.zs.cmd[R300_ZS_CNTL_0] |= R300_RB3D_Z_WRITE_ONLY;
 		r300->hw.zs.cmd[R300_ZS_CNTL_1] |= (R300_ZS_ALWAYS<<R300_RB3D_ZS1_DEPTH_FUNC_SHIFT);
-/*
-		R300_STATECHANGE(r300, zb);
-		r300->hw.zb.cmd[R300_ZB_OFFSET] =
-			1024*4*300 +
-			r300->radeon.radeonScreen->frontOffset +
-			r300->radeon.radeonScreen->fbLocation;
-		r300->hw.zb.cmd[R300_ZB_PITCH] =
-			r300->radeon.radeonScreen->depthPitch;
-*/
 	} else {
-		r300->hw.zs.cmd[R300_ZS_CNTL_0] &= R300_RB3D_STENCIL_ENABLE;
 		r300->hw.zs.cmd[R300_ZS_CNTL_0] |= R300_RB3D_Z_DISABLED_1; // disable
-		r300->hw.zs.cmd[R300_ZS_CNTL_1] &= ~(R300_ZS_MASK << R300_RB3D_ZS1_DEPTH_FUNC_SHIFT);
 	}
 	
 	R300_STATECHANGE(r300, zs);
 	if (flags & CLEARBUFFER_STENCIL) {
-		r300->hw.zs.cmd[R300_ZS_CNTL_0] &= ~R300_RB3D_STENCIL_ENABLE;
 		r300->hw.zs.cmd[R300_ZS_CNTL_0] |= R300_RB3D_STENCIL_ENABLE;
-		r300->hw.zs.cmd[R300_ZS_CNTL_1] &= 
-		    ~((R300_ZS_MASK << R300_RB3D_ZS1_FRONT_FUNC_SHIFT) | (R300_ZS_MASK << R300_RB3D_ZS1_BACK_FUNC_SHIFT));
 		r300->hw.zs.cmd[R300_ZS_CNTL_1] |= 
 		    (R300_ZS_ALWAYS<<R300_RB3D_ZS1_FRONT_FUNC_SHIFT) | 
 		    (R300_ZS_REPLACE<<R300_RB3D_ZS1_FRONT_FAIL_OP_SHIFT) |
@@ -263,7 +249,7 @@ static void r300ClearBuffer(r300ContextPtr r300, int flags, int buffer)
 #else
 #if 1
 	cp_wait(r300, R300_WAIT_3D | R300_WAIT_3D_CLEAN);
-	end_3d(PASS_PREFIX_VOID);
+	end_3d(rmesa);
 #endif
 	
 	R300_STATECHANGE(r300, cb);
@@ -299,37 +285,18 @@ static void r300ClearBuffer(r300ContextPtr r300, int flags, int buffer)
 	{
 	uint32_t t1, t2;
 	
-	t1 = r300->hw.zs.cmd[R300_ZS_CNTL_0];
-	t2 = r300->hw.zs.cmd[R300_ZS_CNTL_1];
+	t1 = 0x0;
+	t2 = 0x0;
 	
 	if (flags & CLEARBUFFER_DEPTH) {
-		t1 &= R300_RB3D_STENCIL_ENABLE;
-		t1 |= 0x6; // test and write
-		
-		t2 &= ~(R300_ZS_MASK << R300_RB3D_ZS1_DEPTH_FUNC_SHIFT);
-		t2 |= (R300_ZS_ALWAYS<<R300_RB3D_ZS1_DEPTH_FUNC_SHIFT);
-/*
-		R300_STATECHANGE(r300, zb);
-		r300->hw.zb.cmd[R300_ZB_OFFSET] =
-			1024*4*300 +
-			r300->radeon.radeonScreen->frontOffset +
-			r300->radeon.radeonScreen->fbLocation;
-		r300->hw.zb.cmd[R300_ZB_PITCH] =
-			r300->radeon.radeonScreen->depthPitch;
-*/
+		t1 |= R300_RB3D_Z_WRITE_ONLY;
+		t2 |= (R300_ZS_ALWAYS << R300_RB3D_ZS1_DEPTH_FUNC_SHIFT);
 	} else {
-		t1 &= R300_RB3D_STENCIL_ENABLE;
 		t1 |= R300_RB3D_Z_DISABLED_1; // disable
-		
-		t2 &= ~(R300_ZS_MASK << R300_RB3D_ZS1_DEPTH_FUNC_SHIFT);
 	}
 	
 	if (flags & CLEARBUFFER_STENCIL) {
-		t1 &= ~R300_RB3D_STENCIL_ENABLE;
 		t1 |= R300_RB3D_STENCIL_ENABLE;
-		
-		t2 &= 
-		    ~((R300_ZS_MASK << R300_RB3D_ZS1_FRONT_FUNC_SHIFT) | (R300_ZS_MASK << R300_RB3D_ZS1_BACK_FUNC_SHIFT));
 		t2 |= 
 		    (R300_ZS_ALWAYS<<R300_RB3D_ZS1_FRONT_FUNC_SHIFT) | 
 		    (R300_ZS_REPLACE<<R300_RB3D_ZS1_FRONT_FAIL_OP_SHIFT) |
@@ -378,12 +345,20 @@ static void r300EmitClearState(GLcontext * ctx)
 	r300ContextPtr rmesa=r300;
 	__DRIdrawablePrivate *dPriv = r300->radeon.dri.drawable;
 	int i;
-	LOCAL_VARS;
+	int cmd_reserved = 0;
+	int cmd_written = 0;
+	drm_radeon_cmd_header_t *cmd = NULL;
+	
 	
 	R300_STATECHANGE(r300, vir[0]);
 	reg_start(R300_VAP_INPUT_ROUTE_0_0, 0);
 	e32(0x21030003);
 	
+	/* disable fog */
+	R300_STATECHANGE(r300, fogs);
+	reg_start(R300_RE_FOG_STATE, 0);
+	e32(0x0);
+	
 	R300_STATECHANGE(r300, vir[1]);
 	reg_start(R300_VAP_INPUT_ROUTE_1_0, 0);
 	e32(0xF688F688);
@@ -502,8 +477,7 @@ static void r300EmitClearState(GLcontext * ctx)
 /**
  * Buffer clear
  */
-static void r300Clear(GLcontext * ctx, GLbitfield mask, GLboolean all,
-		      GLint cx, GLint cy, GLint cw, GLint ch)
+static void r300Clear(GLcontext * ctx, GLbitfield mask)
 {
 	r300ContextPtr r300 = R300_CONTEXT(ctx);
 	__DRIdrawablePrivate *dPriv = r300->radeon.dri.drawable;
@@ -512,8 +486,7 @@ static void r300Clear(GLcontext * ctx, GLbitfield mask, GLboolean all,
 	int swapped;
 
 	if (RADEON_DEBUG & DEBUG_IOCTL)
-		fprintf(stderr, "%s:  all=%d cx=%d cy=%d cw=%d ch=%d\n",
-			__FUNCTION__, all, cx, cy, cw, ch);
+		fprintf(stderr, "r300Clear\n");
 
 	{
 		LOCK_HARDWARE(&r300->radeon);
@@ -546,14 +519,14 @@ static void r300Clear(GLcontext * ctx, GLbitfield mask, GLboolean all,
 		if (RADEON_DEBUG & DEBUG_FALLBACKS)
 			fprintf(stderr, "%s: swrast clear, mask: %x\n",
 				__FUNCTION__, mask);
-		_swrast_Clear(ctx, mask, all, cx, cy, cw, ch);
+		_swrast_Clear(ctx, mask);
 	}
 
 	swapped = r300->radeon.doPageFlip && (r300->radeon.sarea->pfCurrentPage == 1);
 
 #ifdef CB_DPATH
 	/* Make sure it fits there. */
-	r300EnsureCmdBufSpace(r300, 419*3, __FUNCTION__);
+	r300EnsureCmdBufSpace(r300, 421*3, __FUNCTION__);
 	if(flags || bits)
 		r300EmitClearState(ctx);
 #endif
@@ -597,9 +570,10 @@ void r300Flush(GLcontext * ctx)
 #ifdef USER_BUFFERS
 #include "radeon_mm.h"
 
-void r300RefillCurrentDmaRegion(r300ContextPtr rmesa)
+void r300RefillCurrentDmaRegion(r300ContextPtr rmesa, int size)
 {
 	struct r300_dma_buffer *dmabuf;
+	size = MAX2(size, RADEON_BUFFER_SIZE*16);
 	
 	if (RADEON_DEBUG & (DEBUG_IOCTL | DEBUG_DMA))
 		fprintf(stderr, "%s\n", __FUNCTION__);
@@ -618,20 +592,20 @@ void r300RefillCurrentDmaRegion(r300ContextPtr rmesa)
 	dmabuf->buf = (void *)1; /* hack */
 	dmabuf->refcount = 1;
 
-	dmabuf->id = radeon_mm_alloc(rmesa, 4, RADEON_BUFFER_SIZE*16);
+	dmabuf->id = radeon_mm_alloc(rmesa, 4, size);
 	if (dmabuf->id == 0) {
 		LOCK_HARDWARE(&rmesa->radeon);	/* no need to validate */
 		
 		r300FlushCmdBufLocked(rmesa, __FUNCTION__);
 		radeonWaitForIdleLocked(&rmesa->radeon);
 		
-		dmabuf->id = radeon_mm_alloc(rmesa, 4, RADEON_BUFFER_SIZE*16);
+		dmabuf->id = radeon_mm_alloc(rmesa, 4, size);
 
 #ifdef HW_VBOS
 		if (dmabuf->id == 0) {
 			/* Just kick all */
 			r300_evict_vbos(rmesa->radeon.glCtx, /*RADEON_BUFFER_SIZE*16*/1<<30);
-			dmabuf->id = radeon_mm_alloc(rmesa, 4, RADEON_BUFFER_SIZE*16);
+			dmabuf->id = radeon_mm_alloc(rmesa, 4, size);
 		}
 #endif
 		UNLOCK_HARDWARE(&rmesa->radeon);
@@ -644,7 +618,7 @@ void r300RefillCurrentDmaRegion(r300ContextPtr rmesa)
 			
 	rmesa->dma.current.buf = dmabuf;
 	rmesa->dma.current.address = radeon_mm_ptr(rmesa, dmabuf->id);
-	rmesa->dma.current.end = RADEON_BUFFER_SIZE*16;
+	rmesa->dma.current.end = size;
 	rmesa->dma.current.start = 0;
 	rmesa->dma.current.ptr = 0;
 }
@@ -692,7 +666,8 @@ void r300AllocDmaRegion(r300ContextPtr rmesa,
 	    (rmesa->dma.current.ptr + alignment) & ~alignment;
 
 	if (rmesa->dma.current.ptr + bytes > rmesa->dma.current.end)
-		r300RefillCurrentDmaRegion(rmesa);
+		r300RefillCurrentDmaRegion(rmesa,
+					   (bytes + 0x7) & ~0x7);
 
 	region->start = rmesa->dma.current.start;
 	region->ptr = rmesa->dma.current.start;
@@ -869,9 +844,6 @@ GLuint r300GetMemoryOffsetMESA(__DRInativeDisplay * dpy, int scrn,
 	if (!r300IsGartMemory(rmesa, pointer, 0))
 		return ~0;
 
-	if (rmesa->radeon.dri.drmMinor < 6)
-		return ~0;
-
 	card_offset = r300GartOffsetFromVirtual(rmesa, pointer);
 
 	return card_offset - rmesa->radeon.radeonScreen->gart_base;
