@@ -64,50 +64,6 @@ WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #include "drirenderbuffer.h"
 
-static void r300AlphaFunc(GLcontext * ctx, GLenum func, GLfloat ref)
-{
-	r300ContextPtr rmesa = R300_CONTEXT(ctx);
-	int pp_misc = rmesa->hw.at.cmd[R300_AT_ALPHA_TEST];
-	GLubyte refByte;
-
-	CLAMPED_FLOAT_TO_UBYTE(refByte, ref);
-	
-	R300_STATECHANGE(rmesa, at);
-
-	pp_misc &= ~(R300_ALPHA_TEST_OP_MASK | R300_REF_ALPHA_MASK);
-	pp_misc |= (refByte & R300_REF_ALPHA_MASK);
-
-	switch (func) {
-	case GL_NEVER:
-		pp_misc |= R300_ALPHA_TEST_FAIL;
-		break;
-	case GL_LESS:
-		pp_misc |= R300_ALPHA_TEST_LESS;
-		break;
-	case GL_EQUAL:
-		pp_misc |= R300_ALPHA_TEST_EQUAL;
-		break;
-	case GL_LEQUAL:
-		pp_misc |= R300_ALPHA_TEST_LEQUAL;
-		break;
-	case GL_GREATER:
-		pp_misc |= R300_ALPHA_TEST_GREATER;
-		break;
-	case GL_NOTEQUAL:
-		pp_misc |= R300_ALPHA_TEST_NEQUAL;
-		break;
-	case GL_GEQUAL:
-		pp_misc |= R300_ALPHA_TEST_GEQUAL;
-		break;
-	case GL_ALWAYS:
-		pp_misc |= R300_ALPHA_TEST_PASS;
-		//pp_misc &= ~R300_ALPHA_TEST_ENABLE;
-		break;
-	}
-
-	rmesa->hw.at.cmd[R300_AT_ALPHA_TEST] = pp_misc;
-}
-
 static void r300BlendColor(GLcontext * ctx, const GLfloat cf[4])
 {
 	GLubyte color[4];
@@ -144,55 +100,54 @@ static int blend_factor(GLenum factor, GLboolean is_src)
 
 	switch (factor) {
 	case GL_ZERO:
-		func = R200_BLEND_GL_ZERO;
+		func = R300_BLEND_GL_ZERO;
 		break;
 	case GL_ONE:
-		func = R200_BLEND_GL_ONE;
+		func = R300_BLEND_GL_ONE;
 		break;
 	case GL_DST_COLOR:
-		func = R200_BLEND_GL_DST_COLOR;
+		func = R300_BLEND_GL_DST_COLOR;
 		break;
 	case GL_ONE_MINUS_DST_COLOR:
-		func = R200_BLEND_GL_ONE_MINUS_DST_COLOR;
+		func = R300_BLEND_GL_ONE_MINUS_DST_COLOR;
 		break;
 	case GL_SRC_COLOR:
-		func = R200_BLEND_GL_SRC_COLOR;
+		func = R300_BLEND_GL_SRC_COLOR;
 		break;
 	case GL_ONE_MINUS_SRC_COLOR:
-		func = R200_BLEND_GL_ONE_MINUS_SRC_COLOR;
+		func = R300_BLEND_GL_ONE_MINUS_SRC_COLOR;
 		break;
 	case GL_SRC_ALPHA:
-		func = R200_BLEND_GL_SRC_ALPHA;
+		func = R300_BLEND_GL_SRC_ALPHA;
 		break;
 	case GL_ONE_MINUS_SRC_ALPHA:
-		func = R200_BLEND_GL_ONE_MINUS_SRC_ALPHA;
+		func = R300_BLEND_GL_ONE_MINUS_SRC_ALPHA;
 		break;
 	case GL_DST_ALPHA:
-		func = R200_BLEND_GL_DST_ALPHA;
+		func = R300_BLEND_GL_DST_ALPHA;
 		break;
 	case GL_ONE_MINUS_DST_ALPHA:
-		func = R200_BLEND_GL_ONE_MINUS_DST_ALPHA;
+		func = R300_BLEND_GL_ONE_MINUS_DST_ALPHA;
 		break;
 	case GL_SRC_ALPHA_SATURATE:
-		func =
-		    (is_src) ? R200_BLEND_GL_SRC_ALPHA_SATURATE :
-		    R200_BLEND_GL_ZERO;
+		func = (is_src) ? R300_BLEND_GL_SRC_ALPHA_SATURATE :
+		R300_BLEND_GL_ZERO;
 		break;
 	case GL_CONSTANT_COLOR:
-		func = R200_BLEND_GL_CONST_COLOR;
+		func = R300_BLEND_GL_CONST_COLOR;
 		break;
 	case GL_ONE_MINUS_CONSTANT_COLOR:
-		func = R200_BLEND_GL_ONE_MINUS_CONST_COLOR;
+		func = R300_BLEND_GL_ONE_MINUS_CONST_COLOR;
 		break;
 	case GL_CONSTANT_ALPHA:
-		func = R200_BLEND_GL_CONST_ALPHA;
+		func = R300_BLEND_GL_CONST_ALPHA;
 		break;
 	case GL_ONE_MINUS_CONSTANT_ALPHA:
-		func = R200_BLEND_GL_ONE_MINUS_CONST_ALPHA;
+		func = R300_BLEND_GL_ONE_MINUS_CONST_ALPHA;
 		break;
 	default:
 		fprintf(stderr, "unknown blend factor %x\n", factor);
-		func = (is_src) ? R200_BLEND_GL_ONE : R200_BLEND_GL_ZERO;
+		func = (is_src) ? R300_BLEND_GL_ONE : R300_BLEND_GL_ZERO;
 	}
 	return func;
 }
@@ -202,10 +157,10 @@ static int blend_factor(GLenum factor, GLboolean is_src)
  * This is done in a single
  * function because some blend equations (i.e., \c GL_MIN and \c GL_MAX)
  * change the interpretation of the blend function.
- * Also, make sure that blend function and blend equation are set to their default
- * value if color blending is not enabled, since at least blend equations GL_MIN
- * and GL_FUNC_REVERSE_SUBTRACT will cause wrong results otherwise for
- * unknown reasons.
+ * Also, make sure that blend function and blend equation are set to their
+ * default value if color blending is not enabled, since at least blend
+ * equations GL_MIN and GL_FUNC_REVERSE_SUBTRACT will cause wrong results
+ * otherwise for unknown reasons.
  */
 
 /* helper function */
@@ -244,22 +199,22 @@ static void r300_set_blend_cntl(r300ContextPtr r300, int func, int eqn, int cbit
 static void r300_set_blend_state(GLcontext * ctx)
 {
 	r300ContextPtr r300 = R300_CONTEXT(ctx);
-	int func = (R200_BLEND_GL_ONE << R200_SRC_BLEND_SHIFT) |
-	    (R200_BLEND_GL_ZERO << R200_DST_BLEND_SHIFT);
-	int eqn = R200_COMB_FCN_ADD_CLAMP;
-	int funcA = (R200_BLEND_GL_ONE << R200_SRC_BLEND_SHIFT) |
-	    (R200_BLEND_GL_ZERO << R200_DST_BLEND_SHIFT);
-	int eqnA = R200_COMB_FCN_ADD_CLAMP;
+	int func = (R300_BLEND_GL_ONE << R300_SRC_BLEND_SHIFT) |
+	    (R300_BLEND_GL_ZERO << R300_DST_BLEND_SHIFT);
+	int eqn = R300_COMB_FCN_ADD_CLAMP;
+	int funcA = (R300_BLEND_GL_ONE << R300_SRC_BLEND_SHIFT) |
+	    (R300_BLEND_GL_ZERO << R300_DST_BLEND_SHIFT);
+	int eqnA = R300_COMB_FCN_ADD_CLAMP;
 
-	if (ctx->Color._LogicOpEnabled || !ctx->Color.BlendEnabled) {
+	if (RGBA_LOGICOP_ENABLED(ctx) || !ctx->Color.BlendEnabled) {
 		r300_set_blend_cntl(r300,
 			func, eqn, 0,
 			func, eqn);
 		return;
 	}
 
-	func = (blend_factor(ctx->Color.BlendSrcRGB, GL_TRUE) << R200_SRC_BLEND_SHIFT) |
-		(blend_factor(ctx->Color.BlendDstRGB, GL_FALSE) << R200_DST_BLEND_SHIFT);
+	func = (blend_factor(ctx->Color.BlendSrcRGB, GL_TRUE) << R300_SRC_BLEND_SHIFT) |
+		(blend_factor(ctx->Color.BlendDstRGB, GL_FALSE) << R300_DST_BLEND_SHIFT);
 
 	switch (ctx->Color.BlendEquationRGB) {
 	case GL_FUNC_ADD:
@@ -271,19 +226,19 @@ static void r300_set_blend_state(GLcontext * ctx)
 		break;
 
 	case GL_FUNC_REVERSE_SUBTRACT:
-		eqn = R200_COMB_FCN_RSUB_CLAMP;
+		eqn = R300_COMB_FCN_RSUB_CLAMP;
 		break;
 
 	case GL_MIN:
-		eqn = R200_COMB_FCN_MIN;
-		func = (R200_BLEND_GL_ONE << R200_SRC_BLEND_SHIFT) |
-		    (R200_BLEND_GL_ONE << R200_DST_BLEND_SHIFT);
+		eqn = R300_COMB_FCN_MIN;
+		func = (R300_BLEND_GL_ONE << R300_SRC_BLEND_SHIFT) |
+		    (R300_BLEND_GL_ONE << R300_DST_BLEND_SHIFT);
 		break;
 
 	case GL_MAX:
-		eqn = R200_COMB_FCN_MAX;
-		func = (R200_BLEND_GL_ONE << R200_SRC_BLEND_SHIFT) |
-		    (R200_BLEND_GL_ONE << R200_DST_BLEND_SHIFT);
+		eqn = R300_COMB_FCN_MAX;
+		func = (R300_BLEND_GL_ONE << R300_SRC_BLEND_SHIFT) |
+		    (R300_BLEND_GL_ONE << R300_DST_BLEND_SHIFT);
 		break;
 
 	default:
@@ -294,8 +249,8 @@ static void r300_set_blend_state(GLcontext * ctx)
 	}
 
 
-	funcA = (blend_factor(ctx->Color.BlendSrcA, GL_TRUE) << R200_SRC_BLEND_SHIFT) |
-		(blend_factor(ctx->Color.BlendDstA, GL_FALSE) << R200_DST_BLEND_SHIFT);
+	funcA = (blend_factor(ctx->Color.BlendSrcA, GL_TRUE) << R300_SRC_BLEND_SHIFT) |
+		(blend_factor(ctx->Color.BlendDstA, GL_FALSE) << R300_DST_BLEND_SHIFT);
 
 	switch (ctx->Color.BlendEquationA) {
 	case GL_FUNC_ADD:
@@ -307,19 +262,19 @@ static void r300_set_blend_state(GLcontext * ctx)
 		break;
 
 	case GL_FUNC_REVERSE_SUBTRACT:
-		eqnA = R200_COMB_FCN_RSUB_CLAMP;
+		eqnA = R300_COMB_FCN_RSUB_CLAMP;
 		break;
 
 	case GL_MIN:
-		eqnA = R200_COMB_FCN_MIN;
-		funcA = (R200_BLEND_GL_ONE << R200_SRC_BLEND_SHIFT) |
-		    (R200_BLEND_GL_ONE << R200_DST_BLEND_SHIFT);
+		eqnA = R300_COMB_FCN_MIN;
+		funcA = (R300_BLEND_GL_ONE << R300_SRC_BLEND_SHIFT) |
+		    (R300_BLEND_GL_ONE << R300_DST_BLEND_SHIFT);
 		break;
 
 	case GL_MAX:
-		eqnA = R200_COMB_FCN_MAX;
-		funcA = (R200_BLEND_GL_ONE << R200_SRC_BLEND_SHIFT) |
-		    (R200_BLEND_GL_ONE << R200_DST_BLEND_SHIFT);
+		eqnA = R300_COMB_FCN_MAX;
+		funcA = (R300_BLEND_GL_ONE << R300_SRC_BLEND_SHIFT) |
+		    (R300_BLEND_GL_ONE << R300_DST_BLEND_SHIFT);
 		break;
 
 	default:
@@ -371,7 +326,7 @@ static void r300UpdateCulling(GLcontext* ctx)
 	r300->hw.cul.cmd[R300_CUL_CULL] = val;
 }
 
-static void update_early_z(GLcontext* ctx)
+static void update_early_z(GLcontext *ctx)
 {
 	/* updates register 0x4f14 
 	   if depth test is not enabled it should be 0x00000000
@@ -381,11 +336,11 @@ static void update_early_z(GLcontext* ctx)
 	r300ContextPtr r300 = R300_CONTEXT(ctx);
 
 	R300_STATECHANGE(r300, unk4F10);
-	if (ctx->Color.AlphaEnabled)
+	if (ctx->Color.AlphaEnabled && ctx->Color.AlphaFunc != GL_ALWAYS)
 		/* disable early Z */
 		r300->hw.unk4F10.cmd[2] = 0x00000000;
 	else {
-		if (ctx->Depth.Test)
+		if (ctx->Depth.Test && ctx->Depth.Func != GL_NEVER)
 			/* enable early Z */
 			r300->hw.unk4F10.cmd[2] = 0x00000001;
 		else
@@ -394,6 +349,109 @@ static void update_early_z(GLcontext* ctx)
 	}
 }
 
+static void update_alpha(GLcontext *ctx)
+{
+	r300ContextPtr r300 = R300_CONTEXT(ctx);
+	GLubyte refByte;
+	uint32_t pp_misc = 0x0;
+	GLboolean really_enabled = ctx->Color.AlphaEnabled;
+
+	CLAMPED_FLOAT_TO_UBYTE(refByte, ctx->Color.AlphaRef);
+	
+	switch (ctx->Color.AlphaFunc) {
+	case GL_NEVER:
+		pp_misc |= R300_ALPHA_TEST_FAIL;
+		break;
+	case GL_LESS:
+		pp_misc |= R300_ALPHA_TEST_LESS;
+		break;
+	case GL_EQUAL:
+		pp_misc |= R300_ALPHA_TEST_EQUAL;
+		break;
+	case GL_LEQUAL:
+		pp_misc |= R300_ALPHA_TEST_LEQUAL;
+		break;
+	case GL_GREATER:
+		pp_misc |= R300_ALPHA_TEST_GREATER;
+		break;
+	case GL_NOTEQUAL:
+		pp_misc |= R300_ALPHA_TEST_NEQUAL;
+		break;
+	case GL_GEQUAL:
+		pp_misc |= R300_ALPHA_TEST_GEQUAL;
+		break;
+	case GL_ALWAYS:
+		/*pp_misc |= R300_ALPHA_TEST_PASS;*/
+		really_enabled = GL_FALSE;
+		break;
+	}
+	
+	if (really_enabled) {
+		pp_misc |= R300_ALPHA_TEST_ENABLE;
+		pp_misc |= (refByte & R300_REF_ALPHA_MASK);
+	} else {
+		pp_misc = 0x0;
+	}
+	
+	
+	R300_STATECHANGE(r300, at);
+	r300->hw.at.cmd[R300_AT_ALPHA_TEST] = pp_misc;
+	update_early_z(ctx);
+}
+
+static void r300AlphaFunc(GLcontext * ctx, GLenum func, GLfloat ref)
+{
+	(void) func;
+	(void) ref;
+	update_alpha(ctx);
+}
+
+static int translate_func(int func)
+{
+	switch (func) {
+	case GL_NEVER:
+		return R300_ZS_NEVER;
+	case GL_LESS:
+		return R300_ZS_LESS;
+	case GL_EQUAL:
+		return R300_ZS_EQUAL;
+	case GL_LEQUAL:
+		return R300_ZS_LEQUAL;
+	case GL_GREATER:
+		return R300_ZS_GREATER;
+	case GL_NOTEQUAL:
+		return R300_ZS_NOTEQUAL;
+	case GL_GEQUAL:
+		return R300_ZS_GEQUAL;
+	case GL_ALWAYS:
+		return R300_ZS_ALWAYS;
+	}
+	return 0;
+}
+
+static void update_depth(GLcontext* ctx)
+{
+	r300ContextPtr r300 = R300_CONTEXT(ctx);
+
+	R300_STATECHANGE(r300, zs);
+	r300->hw.zs.cmd[R300_ZS_CNTL_0] &= R300_RB3D_STENCIL_ENABLE;
+	r300->hw.zs.cmd[R300_ZS_CNTL_1] &= ~(R300_ZS_MASK << R300_RB3D_ZS1_DEPTH_FUNC_SHIFT);
+	
+	if (ctx->Depth.Test && ctx->Depth.Func != GL_NEVER) {
+		if (ctx->Depth.Mask)
+			r300->hw.zs.cmd[R300_ZS_CNTL_0] |= R300_RB3D_Z_TEST_AND_WRITE;
+		else
+			r300->hw.zs.cmd[R300_ZS_CNTL_0] |= R300_RB3D_Z_TEST;
+		
+		r300->hw.zs.cmd[R300_ZS_CNTL_1] |= translate_func(ctx->Depth.Func) << R300_RB3D_ZS1_DEPTH_FUNC_SHIFT;
+	} else {
+		r300->hw.zs.cmd[R300_ZS_CNTL_0] |= R300_RB3D_Z_DISABLED_1;
+		r300->hw.zs.cmd[R300_ZS_CNTL_1] |= translate_func(GL_NEVER) << R300_RB3D_ZS1_DEPTH_FUNC_SHIFT;
+	}
+	
+	update_early_z(ctx);
+}
+
 /**
  * Handle glEnable()/glDisable().
  *
@@ -402,7 +460,6 @@ static void update_early_z(GLcontext* ctx)
 static void r300Enable(GLcontext* ctx, GLenum cap, GLboolean state)
 {
 	r300ContextPtr r300 = R300_CONTEXT(ctx);
-	uint32_t newval;
 
 	if (RADEON_DEBUG & DEBUG_STATE)
 		fprintf(stderr, "%s( %s = %s )\n", __FUNCTION__,
@@ -417,16 +474,26 @@ static void r300Enable(GLcontext* ctx, GLenum cap, GLboolean state)
 	case GL_TEXTURE_3D:
 		break;
 
-	case GL_ALPHA_TEST:
-		R300_STATECHANGE(r300, at);
+	case GL_FOG:
+		R300_STATECHANGE(r300, fogs);
 		if (state) {
-			r300->hw.at.cmd[R300_AT_ALPHA_TEST] |=
-			    R300_ALPHA_TEST_ENABLE;
+			r300->hw.fogs.cmd[R300_FOGS_STATE] |=
+			    R300_FOG_ENABLE;
+			
+			ctx->Driver.Fogfv( ctx, GL_FOG_MODE, NULL );
+			ctx->Driver.Fogfv( ctx, GL_FOG_DENSITY, &ctx->Fog.Density );
+			ctx->Driver.Fogfv( ctx, GL_FOG_START, &ctx->Fog.Start );
+			ctx->Driver.Fogfv( ctx, GL_FOG_END, &ctx->Fog.End );
+			ctx->Driver.Fogfv( ctx, GL_FOG_COLOR, ctx->Fog.Color );
 		} else {
-			r300->hw.at.cmd[R300_AT_ALPHA_TEST] &=
-			    ~R300_ALPHA_TEST_ENABLE;
+			r300->hw.fogs.cmd[R300_FOGS_STATE] &=
+			    ~R300_FOG_ENABLE;
 		}
-		update_early_z(ctx);
+		
+		break;
+
+	case GL_ALPHA_TEST:
+		update_alpha(ctx);
 		break;
 
 	case GL_BLEND:
@@ -435,19 +502,7 @@ static void r300Enable(GLcontext* ctx, GLenum cap, GLboolean state)
 		break;
 
 	case GL_DEPTH_TEST:
-		R300_STATECHANGE(r300, zs);
-
-		if (state) {
-			if (ctx->Depth.Mask)
-				newval = R300_RB3D_Z_TEST_AND_WRITE;
-			else
-				newval = R300_RB3D_Z_TEST;
-		} else
-			newval = R300_RB3D_Z_DISABLED_1;
-
-		r300->hw.zs.cmd[R300_ZS_CNTL_0] &= R300_RB3D_STENCIL_ENABLE;
-		r300->hw.zs.cmd[R300_ZS_CNTL_0] |= newval;
-		update_early_z(ctx);
+		update_depth(ctx);
 		break;
 
 	case GL_STENCIL_TEST:
@@ -575,38 +630,8 @@ static void r300FrontFace(GLcontext* ctx, GLenum mode)
  */
 static void r300DepthFunc(GLcontext* ctx, GLenum func)
 {
-	r300ContextPtr r300 = R300_CONTEXT(ctx);
-
-	R300_STATECHANGE(r300, zs);
-
-	r300->hw.zs.cmd[R300_ZS_CNTL_1] &= ~(R300_ZS_MASK << R300_RB3D_ZS1_DEPTH_FUNC_SHIFT);
-
-	switch(func) {
-	case GL_NEVER:
-		r300->hw.zs.cmd[R300_ZS_CNTL_1] |= R300_ZS_NEVER << R300_RB3D_ZS1_DEPTH_FUNC_SHIFT;
-		break;
-	case GL_LESS:
-		r300->hw.zs.cmd[R300_ZS_CNTL_1] |= R300_ZS_LESS << R300_RB3D_ZS1_DEPTH_FUNC_SHIFT;
-		break;
-	case GL_EQUAL:
-		r300->hw.zs.cmd[R300_ZS_CNTL_1] |= R300_ZS_EQUAL << R300_RB3D_ZS1_DEPTH_FUNC_SHIFT;
-		break;
-	case GL_LEQUAL:
-		r300->hw.zs.cmd[R300_ZS_CNTL_1] |= R300_ZS_LEQUAL << R300_RB3D_ZS1_DEPTH_FUNC_SHIFT;
-		break;
-	case GL_GREATER:
-		r300->hw.zs.cmd[R300_ZS_CNTL_1] |= R300_ZS_GREATER << R300_RB3D_ZS1_DEPTH_FUNC_SHIFT;
-		break;
-	case GL_NOTEQUAL:
-		r300->hw.zs.cmd[R300_ZS_CNTL_1] |= R300_ZS_NOTEQUAL << R300_RB3D_ZS1_DEPTH_FUNC_SHIFT;
-		break;
-	case GL_GEQUAL:
-		r300->hw.zs.cmd[R300_ZS_CNTL_1] |= R300_ZS_GEQUAL << R300_RB3D_ZS1_DEPTH_FUNC_SHIFT;
-		break;
-	case GL_ALWAYS:
-		r300->hw.zs.cmd[R300_ZS_CNTL_1] |= R300_ZS_ALWAYS << R300_RB3D_ZS1_DEPTH_FUNC_SHIFT;
-		break;
-	}
+	(void) func;
+	update_depth(ctx);
 }
 
 
@@ -617,7 +642,8 @@ static void r300DepthFunc(GLcontext* ctx, GLenum func)
  */
 static void r300DepthMask(GLcontext* ctx, GLboolean mask)
 {
-	r300Enable(ctx, GL_DEPTH_TEST, ctx->Depth.Test);
+	(void) mask;
+	update_depth(ctx);
 }
 
 
@@ -639,6 +665,101 @@ static void r300ColorMask(GLcontext* ctx,
 	}
 }
 
+/* =============================================================
+ * Fog
+ */
+static void r300Fogfv( GLcontext *ctx, GLenum pname, const GLfloat *param )
+{
+	r300ContextPtr r300 = R300_CONTEXT(ctx);
+	union { int i; float f; } fogScale, fogStart;
+	
+	(void) param;
+	
+	fogScale.i = r300->hw.fogp.cmd[R300_FOGP_SCALE];
+	fogStart.i = r300->hw.fogp.cmd[R300_FOGP_START];
+
+	switch (pname) {
+	case GL_FOG_MODE:
+		if (!ctx->Fog.Enabled)
+			return;
+		switch (ctx->Fog.Mode) {
+		case GL_LINEAR:
+			R300_STATECHANGE(r300, fogs);
+			r300->hw.fogs.cmd[R300_FOGS_STATE] =
+				(r300->hw.fogs.cmd[R300_FOGS_STATE] & ~R300_FOG_MODE_MASK) | R300_FOG_MODE_LINEAR;
+
+			if (ctx->Fog.Start == ctx->Fog.End) {
+				fogScale.f = -1.0;
+				fogStart.f = 1.0;
+			}
+			else {
+				fogScale.f = 1.0 / (ctx->Fog.End-ctx->Fog.Start);
+				fogStart.f = -ctx->Fog.Start / (ctx->Fog.End-ctx->Fog.Start);
+			}
+			break;
+		case GL_EXP:
+			R300_STATECHANGE(r300, fogs);
+			r300->hw.fogs.cmd[R300_FOGS_STATE] =
+				(r300->hw.fogs.cmd[R300_FOGS_STATE] & ~R300_FOG_MODE_MASK) | R300_FOG_MODE_EXP;
+			fogScale.f = 0.0933*ctx->Fog.Density;
+			fogStart.f = 0.0;
+			break;
+		case GL_EXP2:
+			R300_STATECHANGE(r300, fogs);
+			r300->hw.fogs.cmd[R300_FOGS_STATE] =
+				(r300->hw.fogs.cmd[R300_FOGS_STATE] & ~R300_FOG_MODE_MASK) | R300_FOG_MODE_EXP2;
+			fogScale.f = 0.3*ctx->Fog.Density;
+			fogStart.f = 0.0;
+		default:
+			return;
+		}
+		break;
+	case GL_FOG_DENSITY:
+		switch (ctx->Fog.Mode) {
+		case GL_EXP:
+			fogScale.f = 0.0933*ctx->Fog.Density;
+			fogStart.f = 0.0;
+			break;
+		case GL_EXP2:
+			fogScale.f = 0.3*ctx->Fog.Density;
+			fogStart.f = 0.0;
+		default:
+			break;
+		}
+		break;
+	case GL_FOG_START:
+	case GL_FOG_END:
+		if (ctx->Fog.Mode == GL_LINEAR) {
+			if (ctx->Fog.Start == ctx->Fog.End) {
+				fogScale.f = -1.0;
+				fogStart.f = 1.0;
+			}
+			else {
+				fogScale.f = 1.0 / (ctx->Fog.End-ctx->Fog.Start);
+				fogStart.f = -ctx->Fog.Start / (ctx->Fog.End-ctx->Fog.Start);
+			}
+		}
+		break;
+	case GL_FOG_COLOR:
+		R300_STATECHANGE(r300, fogc);
+		r300->hw.fogc.cmd[R300_FOGC_R] = (GLuint) (ctx->Fog.Color[0]*1023.0F) & 0x3FF;
+		r300->hw.fogc.cmd[R300_FOGC_G] = (GLuint) (ctx->Fog.Color[1]*1023.0F) & 0x3FF;
+		r300->hw.fogc.cmd[R300_FOGC_B] = (GLuint) (ctx->Fog.Color[2]*1023.0F) & 0x3FF;
+		break;
+	case GL_FOG_COORD_SRC:
+		break;
+	default:
+		return;
+	}
+
+	if (fogScale.i != r300->hw.fogp.cmd[R300_FOGP_SCALE] ||
+	    fogStart.i != r300->hw.fogp.cmd[R300_FOGP_START]) {
+		R300_STATECHANGE(r300, fogp);
+		r300->hw.fogp.cmd[R300_FOGP_SCALE] = fogScale.i;
+		r300->hw.fogp.cmd[R300_FOGP_START] = fogStart.i;
+	}
+}
+
 /* =============================================================
  * Point state
  */
@@ -680,29 +801,6 @@ static void r300PolygonMode(GLcontext *ctx, GLenum face, GLenum mode)
  * Stencil
  */
 
-static int translate_stencil_func(int func)
-{
-	switch (func) {
-	case GL_NEVER:
-		return R300_ZS_NEVER;
-	case GL_LESS:
-		return R300_ZS_LESS;
-	case GL_EQUAL:
-		return R300_ZS_EQUAL;
-	case GL_LEQUAL:
-		return R300_ZS_LEQUAL;
-	case GL_GREATER:
-		return R300_ZS_GREATER;
-	case GL_NOTEQUAL:
-		return R300_ZS_NOTEQUAL;
-	case GL_GEQUAL:
-		return R300_ZS_GEQUAL;
-	case GL_ALWAYS:
-		return R300_ZS_ALWAYS;
-	}
-	return 0;
-}
-
 static int translate_stencil_op(int op)
 {
 	switch (op) {
@@ -750,8 +848,8 @@ static void r300StencilFuncSeparate(GLcontext * ctx, GLenum face,
                                     GLenum func, GLint ref, GLuint mask)
 {
 	r300ContextPtr rmesa = R300_CONTEXT(ctx);
-	GLuint refmask = ((ctx->Stencil.Ref[0] << R300_RB3D_ZS2_STENCIL_REF_SHIFT) |
-			  (ctx->Stencil.ValueMask[0] << R300_RB3D_ZS2_STENCIL_MASK_SHIFT));
+	GLuint refmask = (((ctx->Stencil.Ref[0] & 0xff) << R300_RB3D_ZS2_STENCIL_REF_SHIFT) |
+			  ((ctx->Stencil.ValueMask[0] & 0xff) << R300_RB3D_ZS2_STENCIL_MASK_SHIFT));
 			  
 	GLuint flag;
 
@@ -764,7 +862,7 @@ static void r300StencilFuncSeparate(GLcontext * ctx, GLenum face,
 	rmesa->hw.zs.cmd[R300_ZS_CNTL_2] &=  ~((R300_RB3D_ZS2_STENCIL_MASK << R300_RB3D_ZS2_STENCIL_REF_SHIFT) |
 						(R300_RB3D_ZS2_STENCIL_MASK << R300_RB3D_ZS2_STENCIL_MASK_SHIFT));
 	
-	flag = translate_stencil_func(ctx->Stencil.Function[0]);
+	flag = translate_func(ctx->Stencil.Function[0]);
 
 	rmesa->hw.zs.cmd[R300_ZS_CNTL_1] |= (flag << R300_RB3D_ZS1_FRONT_FUNC_SHIFT)
 					  | (flag << R300_RB3D_ZS1_BACK_FUNC_SHIFT);
@@ -777,7 +875,7 @@ static void r300StencilMaskSeparate(GLcontext * ctx, GLenum face, GLuint mask)
 
 	R300_STATECHANGE(rmesa, zs);
 	rmesa->hw.zs.cmd[R300_ZS_CNTL_2]  &= ~(R300_RB3D_ZS2_STENCIL_MASK << R300_RB3D_ZS2_STENCIL_WRITE_MASK_SHIFT);
-	rmesa->hw.zs.cmd[R300_ZS_CNTL_2] |= ctx->Stencil.WriteMask[0] << R300_RB3D_ZS2_STENCIL_WRITE_MASK_SHIFT;
+	rmesa->hw.zs.cmd[R300_ZS_CNTL_2] |= (ctx->Stencil.WriteMask[0] & 0xff) << R300_RB3D_ZS2_STENCIL_WRITE_MASK_SHIFT;
 }
 
 
@@ -807,9 +905,9 @@ static void r300ClearStencil(GLcontext * ctx, GLint s)
 	r300ContextPtr rmesa = R300_CONTEXT(ctx);
 
 	rmesa->state.stencil.clear =
-	    ((GLuint) ctx->Stencil.Clear |
+	    ((GLuint) (ctx->Stencil.Clear & 0xff) |
 	     (R300_RB3D_ZS2_STENCIL_MASK << R300_RB3D_ZS2_STENCIL_MASK_SHIFT) |
-	     (ctx->Stencil.WriteMask[0] << R300_RB3D_ZS2_STENCIL_WRITE_MASK_SHIFT));
+	     ((ctx->Stencil.WriteMask[0] & 0xff) << R300_RB3D_ZS2_STENCIL_WRITE_MASK_SHIFT));
 }
 
 /* =============================================================
@@ -874,15 +972,15 @@ void r300UpdateViewportOffset( GLcontext *ctx )
 	GLfloat tx = v[MAT_TX] + xoffset + SUBPIXEL_X;
 	GLfloat ty = (- v[MAT_TY]) + yoffset + SUBPIXEL_Y;
 
-	if ( rmesa->hw.vpt.cmd[VPT_SE_VPORT_XOFFSET] != r300PackFloat32(tx) ||
-		rmesa->hw.vpt.cmd[VPT_SE_VPORT_YOFFSET] != r300PackFloat32(ty))
+	if ( rmesa->hw.vpt.cmd[R300_VPT_XOFFSET] != r300PackFloat32(tx) ||
+		rmesa->hw.vpt.cmd[R300_VPT_YOFFSET] != r300PackFloat32(ty))
 	{
 	/* Note: this should also modify whatever data the context reset
 	 * code uses...
 	 */
 	R300_STATECHANGE( rmesa, vpt );
-	rmesa->hw.vpt.cmd[VPT_SE_VPORT_XOFFSET] = r300PackFloat32(tx);
-	rmesa->hw.vpt.cmd[VPT_SE_VPORT_YOFFSET] = r300PackFloat32(ty);
+	rmesa->hw.vpt.cmd[R300_VPT_XOFFSET] = r300PackFloat32(tx);
+	rmesa->hw.vpt.cmd[R300_VPT_YOFFSET] = r300PackFloat32(ty);
       
 	}
 
@@ -946,6 +1044,59 @@ r300UpdateDrawBuffer(GLcontext *ctx)
 #endif
 }
 
+static void r300FetchStateParameter(GLcontext *ctx, const enum state_index state[],
+                  GLfloat *value)
+{
+    r300ContextPtr r300 = R300_CONTEXT(ctx);
+
+    switch(state[0])
+    {
+    case STATE_INTERNAL:
+    	switch(state[1])
+	{
+	case STATE_R300_WINDOW_DIMENSION:
+	    value[0] = r300->radeon.dri.drawable->w;	/* width */
+    	    value[1] = r300->radeon.dri.drawable->h;	/* height */
+	    value[2] = 0.5F; 				/* for moving range [-1 1] -> [0 1] */
+    	    value[3] = 1.0F; 				/* not used */
+	    break;
+	default:;
+	}
+    default:;
+    }
+}
+
+/**
+ * Update R300's own internal state parameters.
+ * For now just STATE_R300_WINDOW_DIMENSION
+ */
+static void r300UpdateStateParameters(GLcontext * ctx, GLuint new_state)
+{
+	struct r300_vertex_program_cont *vpc;
+	struct gl_program_parameter_list *paramList;
+	GLuint i;
+
+	if(!(new_state & (_NEW_BUFFERS|_NEW_PROGRAM)))
+	    return;
+
+	vpc = (struct r300_vertex_program_cont *)ctx->VertexProgram._Current;
+	if (!vpc)
+	    return;
+
+	paramList = vpc->mesa_program.Base.Parameters;
+
+	if (!paramList)
+	    return;
+
+	for (i = 0; i < paramList->NumParameters; i++) {
+		if (paramList->Parameters[i].Type == PROGRAM_STATE_VAR){
+			r300FetchStateParameter(ctx,
+				    paramList->Parameters[i].StateIndexes,
+				    paramList->ParameterValues[i]);
+		}
+	}
+}
+
 /* =============================================================
  * Polygon state
  */
@@ -1157,6 +1308,15 @@ void r300_setup_textures(GLcontext *ctx)
 		fprintf(stderr, "TX_ENABLE: %08x  last_hw_tmu=%d\n", r300->hw.txe.cmd[R300_TXE_ENABLE], last_hw_tmu);
 }
 
+union r300_outputs_written {
+	GLuint vp_outputs;                       /* hw_tcl_on */
+	DECLARE_RENDERINPUTS(index_bitset);      /* !hw_tcl_on */
+};
+
+#define R300_OUTPUTS_WRITTEN_TEST(ow, vp_result, tnl_attrib) \
+	((hw_tcl_on) ? (ow).vp_outputs & (1 << (vp_result)) : \
+	RENDERINPUTS_TEST( (ow.index_bitset), (tnl_attrib) ))
+
 void r300_setup_rs_unit(GLcontext *ctx)
 {
 	r300ContextPtr r300 = R300_CONTEXT(ctx);
@@ -1171,16 +1331,16 @@ void r300_setup_rs_unit(GLcontext *ctx)
 		0x00,
 		0x00
 	};
-	GLuint OutputsWritten;
+	union r300_outputs_written OutputsWritten;
 	GLuint InputsRead;
 	int fp_reg, high_rr;
 	int in_texcoords, col_interp_nr;
 	int i;
 
 	if(hw_tcl_on)
-		OutputsWritten = CURRENT_VERTEX_SHADER(ctx)->Base.OutputsWritten;
+		OutputsWritten.vp_outputs = CURRENT_VERTEX_SHADER(ctx)->key.OutputsWritten;
 	else
-		OutputsWritten = r300->state.render_inputs;
+		RENDERINPUTS_COPY( OutputsWritten.index_bitset, r300->state.render_inputs_bitset );
 
 	if (ctx->FragmentProgram._Current)
 		InputsRead = ctx->FragmentProgram._Current->Base.InputsRead;
@@ -1194,24 +1354,39 @@ void r300_setup_rs_unit(GLcontext *ctx)
 	R300_STATECHANGE(r300, rr);
 	
 	fp_reg = in_texcoords = col_interp_nr = high_rr = 0;
-	r300->hw.rr.cmd[R300_RR_ROUTE_0] = 0;
-	r300->hw.rr.cmd[R300_RR_ROUTE_1] = 0;
 
+	r300->hw.rr.cmd[R300_RR_ROUTE_1] = 0;
+	
+	if (InputsRead & FRAG_BIT_WPOS){
+		for (i = 0; i < ctx->Const.MaxTextureUnits; i++)
+			if (!(InputsRead & (FRAG_BIT_TEX0 << i)))
+				break;
+
+		if(i == ctx->Const.MaxTextureUnits){
+			fprintf(stderr, "\tno free texcoord found...\n");
+			exit(0);
+		}
+
+		InputsRead |= (FRAG_BIT_TEX0 << i);
+		InputsRead &= ~FRAG_BIT_WPOS;
+	}
+	
 	for (i=0;i<ctx->Const.MaxTextureUnits;i++) {
 		r300->hw.ri.cmd[R300_RI_INTERP_0+i] = 0
 				| R300_RS_INTERP_USED
 				| (in_texcoords << R300_RS_INTERP_SRC_SHIFT)
 				| interp_magic[i];
 
+		r300->hw.rr.cmd[R300_RR_ROUTE_0 + fp_reg] = 0;
 		if (InputsRead & (FRAG_BIT_TEX0<<i)) {
 			//assert(r300->state.texture.tc_count != 0);
-			r300->hw.rr.cmd[R300_RR_ROUTE_0 + fp_reg] = 0
-					| R300_RS_ROUTE_ENABLE
+			r300->hw.rr.cmd[R300_RR_ROUTE_0 + fp_reg] |=
+					  R300_RS_ROUTE_ENABLE
 					| i /* source INTERP */
 					| (fp_reg << R300_RS_ROUTE_DEST_SHIFT);
 			high_rr = fp_reg;
 
-			if (!(OutputsWritten & (hw_tcl_on ? (1 << (VERT_RESULT_TEX0+i)) : (_TNL_BIT_TEX0<<i)))) {
+			if (!R300_OUTPUTS_WRITTEN_TEST( OutputsWritten, VERT_RESULT_TEX0+i, _TNL_ATTRIB_TEX(i) )) {
 				/* Passing invalid data here can lock the GPU. */
 				WARN_ONCE("fragprog wants coords for tex%d, vp doesn't provide them!\n", i);
 				//_mesa_print_program(&CURRENT_VERTEX_SHADER(ctx)->Base);
@@ -1221,12 +1396,12 @@ void r300_setup_rs_unit(GLcontext *ctx)
 			fp_reg++;
 		} 
 		/* Need to count all coords enabled at vof */
-		if (OutputsWritten & (hw_tcl_on ? (1 << (VERT_RESULT_TEX0+i)) : (_TNL_BIT_TEX0<<i)))
+		if (R300_OUTPUTS_WRITTEN_TEST( OutputsWritten, VERT_RESULT_TEX0+i, _TNL_ATTRIB_TEX(i) ))
 			in_texcoords++;
 	}
 
 	if (InputsRead & FRAG_BIT_COL0) {
-		if (!(OutputsWritten & (hw_tcl_on ? (1<<VERT_RESULT_COL0) : _TNL_BIT_COLOR0))) {
+		if (!R300_OUTPUTS_WRITTEN_TEST( OutputsWritten, VERT_RESULT_COL0, _TNL_ATTRIB_COLOR0 )) {
 			WARN_ONCE("fragprog wants col0, vp doesn't provide it\n");
 			goto out; /* FIXME */
 			//_mesa_print_program(&CURRENT_VERTEX_SHADER(ctx)->Base);
@@ -1242,7 +1417,7 @@ void r300_setup_rs_unit(GLcontext *ctx)
 	out:
 	
 	if (InputsRead & FRAG_BIT_COL1) {
-		if (!(OutputsWritten & (hw_tcl_on ? (1<<VERT_RESULT_COL1) : _TNL_BIT_COLOR1))) {
+		if (!R300_OUTPUTS_WRITTEN_TEST( OutputsWritten, VERT_RESULT_COL1, _TNL_ATTRIB_COLOR1 )) {
 			WARN_ONCE("fragprog wants col1, vp doesn't provide it\n");
 			//exit(-1);
 		}
@@ -1281,6 +1456,7 @@ void r300_setup_rs_unit(GLcontext *ctx)
 #define bump_vpu_count(ptr, new_count)   do{\
 	drm_r300_cmd_header_t* _p=((drm_r300_cmd_header_t*)(ptr));\
 	int _nc=(new_count)/4; \
+	assert(_nc < 256); \
 	if(_nc>_p->vpu.count)_p->vpu.count=_nc;\
 	}while(0)
 
@@ -1402,26 +1578,16 @@ static void r300GenerateSimpleVertexShader(r300ContextPtr r300)
 		)
 	o_reg += 2;
 	
-	if (r300->state.render_inputs & _TNL_BIT_COLOR1) {
-		WRITE_OP(
-			EASY_VSF_OP(MUL, o_reg++, ALL, RESULT),
-			VSF_REG(r300->state.vap_reg.i_color[1]),
-			VSF_ATTR_UNITY(r300->state.vap_reg.i_color[1]),
-			VSF_UNITY(r300->state.vap_reg.i_color[1])
-		)
-	}
-	
-	/* Pass through texture coordinates, if any */
-	for(i=0;i < r300->radeon.glCtx->Const.MaxTextureUnits;i++)
-		if(r300->state.render_inputs & (_TNL_BIT_TEX0<<i)){
-			// fprintf(stderr, "i_tex[%d]=%d\n", i, r300->state.vap_reg.i_tex[i]);
+	for (i = VERT_ATTRIB_COLOR1; i < VERT_ATTRIB_MAX; i++)
+		if (r300->state.sw_tcl_inputs[i] != -1) {
 			WRITE_OP(
 				EASY_VSF_OP(MUL, o_reg++ /* 2+i */, ALL, RESULT),
-				VSF_REG(r300->state.vap_reg.i_tex[i]),
-				VSF_ATTR_UNITY(r300->state.vap_reg.i_tex[i]),
-				VSF_UNITY(r300->state.vap_reg.i_tex[i])
+				VSF_REG(r300->state.sw_tcl_inputs[i]),
+				VSF_ATTR_UNITY(r300->state.sw_tcl_inputs[i]),
+				VSF_UNITY(r300->state.sw_tcl_inputs[i])
 				)
-			}
+		
+		}
 	
 	r300->state.vertex_shader.program_end--; /* r300 wants program length to be one more - no idea why */
 	r300->state.vertex_shader.program.length=(r300->state.vertex_shader.program_end+1)*4;
@@ -1511,7 +1677,7 @@ void r300SetupVertexProgram(r300ContextPtr rmesa)
 
 	((drm_r300_cmd_header_t*)rmesa->hw.vpp.cmd)->vpu.count = 0;
 	R300_STATECHANGE(rmesa, vpp);
-	param_count = r300VertexProgUpdateParams(ctx, prog, (float *)&rmesa->hw.vpp.cmd[R300_VPP_PARAM_0]);
+	param_count = r300VertexProgUpdateParams(ctx, (struct r300_vertex_program_cont *)ctx->VertexProgram._Current/*prog*/, (float *)&rmesa->hw.vpp.cmd[R300_VPP_PARAM_0]);
 	bump_vpu_count(rmesa->hw.vpp.cmd, param_count);
 	param_count /= 4;
 	
@@ -1552,21 +1718,28 @@ void r300UpdateShaders(r300ContextPtr rmesa)
 {
 	GLcontext *ctx;
 	struct r300_vertex_program *vp;
+	int i;
 	
 	ctx = rmesa->radeon.glCtx;
 	
-	/* Disable tnl programs when doing software vertex programs.
-	   I can only hope this actually disables it at the right time. */
-	ctx->_MaintainTnlProgram = hw_tcl_on;
-	
 	if (rmesa->NewGLState && hw_tcl_on) {
 		rmesa->NewGLState = 0;
 		
+		for (i = _TNL_FIRST_MAT; i <= _TNL_LAST_MAT; i++) {
+			rmesa->temp_attrib[i] = TNL_CONTEXT(ctx)->vb.AttribPtr[i];
+			TNL_CONTEXT(ctx)->vb.AttribPtr[i] = &rmesa->dummy_attrib[i];
+		}
+		
 		_tnl_UpdateFixedFunctionProgram(ctx);
 	
+		for (i = _TNL_FIRST_MAT; i <= _TNL_LAST_MAT; i++) {
+			TNL_CONTEXT(ctx)->vb.AttribPtr[i] = rmesa->temp_attrib[i];
+		}
+		
+		r300_select_vertex_shader(rmesa);
 		vp = (struct r300_vertex_program *)CURRENT_VERTEX_SHADER(ctx);
-		if (vp->translated == GL_FALSE)
-			r300_translate_vertex_shader(vp);
+		/*if (vp->translated == GL_FALSE)
+			r300_translate_vertex_shader(vp);*/
 		if (vp->translated == GL_FALSE) {
 			fprintf(stderr, "Failing back to sw-tcl\n");
 			hw_tcl_on = future_hw_tcl_on = 0;
@@ -1574,6 +1747,7 @@ void r300UpdateShaders(r300ContextPtr rmesa)
 
 			return ;
 		}
+		r300UpdateStateParameters(ctx, _NEW_PROGRAM);
 	}
 	
 }
@@ -1707,6 +1881,9 @@ static void r300InvalidateState(GLcontext * ctx, GLuint new_state)
 	if (new_state & (_NEW_BUFFERS | _NEW_COLOR | _NEW_PIXEL)) {
 		r300UpdateDrawBuffer(ctx);
 	}
+
+	r300UpdateStateParameters(ctx, new_state);
+
 #ifndef CB_DPATH
 	/* Go inefficiency! */
 	r300ResetHwState(r300);
@@ -1804,11 +1981,10 @@ void r300ResetHwState(r300ContextPtr r300)
 
 	r300->hw.unk2134.cmd[1] = 0x00FFFFFF;
 	r300->hw.unk2134.cmd[2] = 0x00000000;
-#ifdef MESA_BIG_ENDIAN
-	r300->hw.unk2140.cmd[1] = 0x00000002;
-#else
-	r300->hw.unk2140.cmd[1] = 0x00000000;
-#endif
+	if (_mesa_little_endian())
+		r300->hw.unk2140.cmd[1] = 0x00000000;
+	else
+		r300->hw.unk2140.cmd[1] = 0x00000002;
 
 #if 0 /* Done in setup routing */
 	((drm_r300_cmd_header_t*)r300->hw.vir[0].cmd)->packet0.count = 1;
@@ -1870,7 +2046,8 @@ void r300ResetHwState(r300ContextPtr r300)
 		r300->hw.gb_misc.cmd[R300_GB_MISC_TILE_CONFIG] = R300_GB_TILE_ENABLE
 							| R300_GB_TILE_PIPE_COUNT_RV300
 							| R300_GB_TILE_SIZE_16;
-	r300->hw.gb_misc.cmd[R300_GB_MISC_SELECT] = 0x00000000;
+	/* set to 0 when fog is disabled? */
+	r300->hw.gb_misc.cmd[R300_GB_MISC_SELECT] = R300_GB_FOG_SELECT_1_1_W;
 	r300->hw.gb_misc.cmd[R300_GB_MISC_AA_CONFIG] = 0x00000000; /* No antialiasing */
 
 	//r300->hw.txe.cmd[R300_TXE_ENABLE] = 0;
@@ -1907,9 +2084,6 @@ void r300ResetHwState(r300ContextPtr r300)
 	r300PolygonMode(ctx, GL_BACK, ctx->Polygon.BackMode);
 	r300->hw.unk4288.cmd[2] = 0x00000001;
 	r300->hw.unk4288.cmd[3] = 0x00000000;
-	r300->hw.unk4288.cmd[4] = 0x00000000;
-	r300->hw.unk4288.cmd[5] = 0x00000000;
-
 	r300->hw.unk42A0.cmd[1] = 0x00000000;
 
 	r300PolygonOffset(ctx, ctx->Polygon.OffsetFactor, ctx->Polygon.OffsetUnits);
@@ -1949,13 +2123,13 @@ void r300ResetHwState(r300ContextPtr r300)
 		r300->hw.fpi[3].cmd[i] = FP_SELA(0,W,NO,FP_TMP(0),0,0);
 	}
 #endif
-
-	r300->hw.unk4BC0.cmd[1] = 0;
-
-	r300->hw.unk4BC8.cmd[1] = 0;
-	r300->hw.unk4BC8.cmd[2] = 0;
-	r300->hw.unk4BC8.cmd[3] = 0;
-
+	r300Enable(ctx, GL_FOG, ctx->Fog.Enabled);
+	ctx->Driver.Fogfv( ctx, GL_FOG_MODE, NULL );
+	ctx->Driver.Fogfv( ctx, GL_FOG_DENSITY, &ctx->Fog.Density );
+	ctx->Driver.Fogfv( ctx, GL_FOG_START, &ctx->Fog.Start );
+	ctx->Driver.Fogfv( ctx, GL_FOG_END, &ctx->Fog.End );
+	ctx->Driver.Fogfv( ctx, GL_FOG_COLOR, ctx->Fog.Color );
+	ctx->Driver.Fogfv( ctx, GL_FOG_COORDINATE_SOURCE_EXT, NULL );
 
 	r300->hw.at.cmd[R300_AT_UNKNOWN] = 0;
 	r300->hw.unk4BD8.cmd[1] = 0;
@@ -2081,12 +2255,12 @@ void r300InitState(r300ContextPtr r300)
 	switch (ctx->Visual.depthBits) {
 	case 16:
 		r300->state.depth.scale = 1.0 / (GLfloat) 0xffff;
-		depth_fmt = R200_DEPTH_FORMAT_16BIT_INT_Z;
+		depth_fmt = R300_DEPTH_FORMAT_16BIT_INT_Z;
 		r300->state.stencil.clear = 0x00000000;
 		break;
 	case 24:
 		r300->state.depth.scale = 1.0 / (GLfloat) 0xffffff;
-		depth_fmt = R200_DEPTH_FORMAT_24BIT_INT_Z;
+		depth_fmt = R300_DEPTH_FORMAT_24BIT_INT_Z;
 		r300->state.stencil.clear = 0x00ff0000;
 		break;
 	default:
@@ -2128,6 +2302,7 @@ void r300InitStateFuncs(struct dd_function_table* functions)
 	functions->DepthFunc = r300DepthFunc;
 	functions->DepthMask = r300DepthMask;
 	functions->CullFace = r300CullFace;
+	functions->Fogfv = r300Fogfv;
 	functions->FrontFace = r300FrontFace;
 	functions->ShadeModel = r300ShadeModel;
 
