@@ -251,7 +251,7 @@ sisDDDepthMask( GLcontext * ctx, GLboolean flag )
    if (ctx->Visual.stencilBits) {
       if (flag || (ctx->Stencil.WriteMask[0] != 0)) {
          current->hwCapEnable |= MASK_ZWriteEnable;
-         if (flag && (ctx->Stencil.WriteMask[0] == 0xff)) {
+         if (flag && ((ctx->Stencil.WriteMask[0] & 0xff) == 0xff)) {
 	      current->hwCapEnable2 &= ~MASK_ZMaskWriteEnable;
          } else {
             current->hwCapEnable2 |= MASK_ZMaskWriteEnable;
@@ -861,18 +861,6 @@ void sisDDInitStateFuncs( GLcontext *ctx )
    ctx->Driver.LightModelfv	 = sisDDLightModelfv;
    ctx->Driver.Viewport		 = sisDDViewport;
 
-   /* Pixel path fallbacks. */
-   ctx->Driver.Accum		 = _swrast_Accum;
-   ctx->Driver.Bitmap		 = _swrast_Bitmap;
-   ctx->Driver.CopyPixels	 = _swrast_CopyPixels;
-   ctx->Driver.DrawPixels	 = _swrast_DrawPixels;
-   ctx->Driver.ReadPixels	 = _swrast_ReadPixels;
-
+   /* XXX this should go away */
    ctx->Driver.ResizeBuffers	 = sisReAllocateBuffers;
-  /* Swrast hooks for imaging extensions:
-   */
-  ctx->Driver.CopyColorTable	 = _swrast_CopyColorTable;
-  ctx->Driver.CopyColorSubTable	 = _swrast_CopyColorSubTable;
-  ctx->Driver.CopyConvolutionFilter1D = _swrast_CopyConvolutionFilter1D;
-  ctx->Driver.CopyConvolutionFilter2D = _swrast_CopyConvolutionFilter2D;
 }
