@@ -23,19 +23,14 @@
  * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE.
  */
-/* $XFree86: xc/lib/GL/mesa/src/drv/tdfx/tdfx_context.c,v 1.12 2003/05/08 09:25:35 herrb Exp $ */
 
-/*
- * New fixes:
- *	Daniel Borca <dborca@users.sourceforge.net>, 19 Jul 2004
- *
- * Original rewrite:
- *	Gareth Hughes <gareth@valinux.com>, 29 Sep - 1 Oct 2000
- *
- * Authors:
- *	Gareth Hughes <gareth@valinux.com>
- *	Brian Paul <brianp@valinux.com>
+/**
+ * \file tdfx_context.c
+ * Context management functions for 3Dfx hardware.
  *
+ * \author Gareth Hughes <gareth@valinux.com> (original rewrite 29 Sep - 1 Oct 2000)
+ * \author Brian Paul <brianp@valinux.com>
+ * \author Daniel Borca <dborca@users.sourceforge.net> (new fixes 19 Jul 2004)
  */
 
 #include <dlfcn.h>
@@ -50,6 +45,7 @@
 #include "tdfx_texman.h"
 #include "extensions.h"
 #include "hash.h"
+#include "texobj.h"
 
 #include "swrast/swrast.h"
 #include "swrast_setup/swrast_setup.h"
@@ -64,6 +60,7 @@
 
 #define need_GL_ARB_multisample
 /* #define need_GL_ARB_point_parameters */
+#define need_GL_ARB_occlusion_query
 #define need_GL_ARB_texture_compression
 #define need_GL_ARB_vertex_buffer_object
 /* #define need_GL_ARB_vertex_program */
@@ -86,6 +83,7 @@
 const struct dri_extension card_extensions[] =
 {
     { "GL_ARB_multisample",                GL_ARB_multisample_functions },
+    { "GL_ARB_occlusion_query",            GL_ARB_occlusion_query_functions },
     { "GL_ARB_texture_mirrored_repeat",    NULL },
     { "GL_ARB_vertex_buffer_object",       GL_ARB_vertex_buffer_object_functions },
 
@@ -97,7 +95,6 @@ const struct dri_extension card_extensions[] =
     { "GL_EXT_stencil_wrap",               NULL },
     { "GL_EXT_texture_env_add",            NULL },
     { "GL_EXT_texture_lod_bias",           NULL },
-    { "GL_HP_occlusion_test",              NULL },
     { "GL_IBM_multimode_draw_arrays",      GL_IBM_multimode_draw_arrays_functions },
 
 #ifdef need_GL_ARB_point_parameters
@@ -603,7 +600,7 @@ tdfxDestroyContext( __DRIcontextPrivate *driContextPriv )
               id;
               id = _mesa_HashNextEntry(textures, id)) {
             struct gl_texture_object *tObj
-               = (struct gl_texture_object *) _mesa_HashLookup(textures, id);
+               = _mesa_lookup_texture(fxMesa->glCtx, id);
             tdfxTMFreeTexture(fxMesa, tObj);
          }
       }
@@ -660,8 +657,10 @@ tdfxMakeCurrent( __DRIcontextPrivate *driContextPriv,
       GLcontext *newCtx = newFx->glCtx;
       GET_CURRENT_CONTEXT(curCtx);
 
-      if ( newFx->driDrawable != driDrawPriv ) {
+      if ((newFx->driDrawable != driDrawPriv)
+	  || (newFx->driReadable != driReadPriv)) {
 	 newFx->driDrawable = driDrawPriv;
+	 newFx->driReadable = driReadPriv;
 	 newFx->dirty = ~0;
       }
       else {
@@ -679,6 +678,11 @@ tdfxMakeCurrent( __DRIcontextPrivate *driContextPriv,
 	 newFx->dirty = ~0;
       }
 
+      driUpdateFramebufferSize(newCtx, driDrawPriv);
+      if (driDrawPriv != driReadPriv) {
+	 driUpdateFramebufferSize(newCtx, driReadPriv);
+      }
+
       if ( !newFx->Glide.Initialized ) {
 	 if ( !tdfxInitContext( driDrawPriv, newFx ) )
 	    return GL_FALSE;
