@@ -281,13 +281,13 @@ static void clear_color(GLcontext *ctx, const GLfloat color[4])
  * Clearing of the other non-color buffers is left to the swrast. 
  */ 
 
-static void clear(GLcontext *ctx, 
-		  GLbitfield mask, 
-		  GLboolean all, 
-		  GLint x, GLint y, 
-		  GLint width, GLint height) 
+static void clear(GLcontext *ctx, GLbitfield mask)
 {
 #define FLIP(Y)  (ctx->DrawBuffer->Height - (Y) - 1)
+    const GLint x = ctx->DrawBuffer->_Xmin;
+    const GLint y = ctx->DrawBuffer->_Ymin;
+    const GLint height = ctx->DrawBuffer->_Ymax - ctx->DrawBuffer->_Ymin;
+    const GLint width  = ctx->DrawBuffer->_Xmax - ctx->DrawBuffer->_Xmin;
 
     WMesaContext pwc = wmesa_context(ctx);
     WMesaFramebuffer pwfb = wmesa_framebuffer(ctx->DrawBuffer);
@@ -299,7 +299,7 @@ static void clear(GLcontext *ctx,
 	ctx->Color.ColorMask[1] != 0xff ||
 	ctx->Color.ColorMask[2] != 0xff ||
 	ctx->Color.ColorMask[3] != 0xff) {
-	_swrast_Clear(ctx, mask, all, x, y, width, height); 
+	_swrast_Clear(ctx, mask);
 	return;
     }
 
@@ -318,7 +318,8 @@ static void clear(GLcontext *ctx,
 	
 	/* Try for a fast clear - clearing entire buffer with a single
 	 * byte value. */
-	if (all) { /* entire buffer */
+	if (width == ctx->DrawBuffer->Width &&
+            height == ctx->DrawBuffer->Height) { /* entire buffer */
 	    /* Now check for an easy clear value */
 	    switch (bytesPerPixel) {
 	    case 1:
@@ -431,7 +432,7 @@ static void clear(GLcontext *ctx,
     
     /* Call swrast if there is anything left to clear (like DEPTH) */ 
     if (mask) 
-	_swrast_Clear(ctx, mask, all, x, y, width, height); 
+	_swrast_Clear(ctx, mask);
   
 #undef FLIP
 } 
@@ -782,7 +783,7 @@ static void read_rgba_pixels_32(const GLcontext *ctx,
     for (i=0; i<n; i++) {
 	GLint y2 = FLIP(y[i]);
 	lpdw = ((LPDWORD)(pwfb->pbPixels + pwfb->ScanWidth * y2)) + x[i];
-	pixel = lpdw[i];
+	pixel = *lpdw;
 	rgba[i][RCOMP] = (pixel & 0x00ff0000) >> 16;
 	rgba[i][GCOMP] = (pixel & 0x0000ff00) >> 8;
 	rgba[i][BCOMP] = (pixel & 0x000000ff);
@@ -966,7 +967,7 @@ static void read_rgba_pixels_16(const GLcontext *ctx,
     for (i=0; i<n; i++) {
 	GLint y2 = FLIP(y[i]);
 	lpw = ((LPWORD)(pwfb->pbPixels + pwfb->ScanWidth * y2)) + x[i];
-	pixel = lpw[i];
+	pixel = *lpw;
 	/* Windows uses 5,5,5 for 16-bit */
 	rgba[i][RCOMP] = (pixel & 0x7c00) >> 7;
 	rgba[i][GCOMP] = (pixel & 0x03e0) >> 2;
@@ -1321,7 +1322,7 @@ void WMesaMakeCurrent(WMesaContext c, HDC hdc)
         /* return if already current */
         GET_CURRENT_CONTEXT(ctx);
         WMesaContext pwc = wmesa_context(ctx);
-        if (c == pwc && pwc->hDC == hdc)
+        if (pwc && c == pwc && pwc->hDC == hdc)
             return;
     }
 
@@ -1397,3 +1398,53 @@ void WMesaSwapBuffers( HDC hdc )
         _mesa_problem(NULL, "wmesa: can't swap non-current window");
     }
 }
+
+/* This is hopefully a temporary hack to define some needed dispatch
+ * table entries.  Hopefully, I'll find a better solution.  The
+ * dispatch table generation scripts ought to be making these dummy
+ * stubs as well. */
+void gl_dispatch_stub_543(void){};
+void gl_dispatch_stub_544(void){};
+void gl_dispatch_stub_545(void){};
+void gl_dispatch_stub_546(void){};
+void gl_dispatch_stub_547(void){};
+void gl_dispatch_stub_548(void){};
+void gl_dispatch_stub_549(void){};
+void gl_dispatch_stub_550(void){};
+void gl_dispatch_stub_551(void){};
+void gl_dispatch_stub_552(void){};
+void gl_dispatch_stub_553(void){};
+void gl_dispatch_stub_554(void){};
+void gl_dispatch_stub_555(void){};
+void gl_dispatch_stub_556(void){};
+void gl_dispatch_stub_557(void){};
+void gl_dispatch_stub_558(void){};
+void gl_dispatch_stub_559(void){};
+void gl_dispatch_stub_560(void){};
+void gl_dispatch_stub_561(void){};
+void gl_dispatch_stub_565(void){};
+void gl_dispatch_stub_566(void){};
+void gl_dispatch_stub_577(void){};
+void gl_dispatch_stub_578(void){};
+void gl_dispatch_stub_603(void){};
+void gl_dispatch_stub_645(void){};
+void gl_dispatch_stub_646(void){};
+void gl_dispatch_stub_647(void){};
+void gl_dispatch_stub_648(void){};
+void gl_dispatch_stub_649(void){};
+void gl_dispatch_stub_650(void){};
+void gl_dispatch_stub_651(void){};
+void gl_dispatch_stub_652(void){};
+void gl_dispatch_stub_653(void){};
+void gl_dispatch_stub_734(void){};
+void gl_dispatch_stub_735(void){};
+void gl_dispatch_stub_736(void){};
+void gl_dispatch_stub_737(void){};
+void gl_dispatch_stub_738(void){};
+void gl_dispatch_stub_745(void){};
+void gl_dispatch_stub_746(void){};
+void gl_dispatch_stub_760(void){};
+void gl_dispatch_stub_761(void){};
+void gl_dispatch_stub_766(void){};
+void gl_dispatch_stub_767(void){};
+void gl_dispatch_stub_768(void){};
