@@ -1,8 +1,8 @@
 /*
  * Mesa 3-D graphics library
- * Version:  6.5
+ * Version:  6.5.1
  *
- * Copyright (C) 1999-2005  Brian Paul   All Rights Reserved.
+ * Copyright (C) 1999-2006  Brian Paul   All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -28,48 +28,39 @@
 #include "api_validate.h"
 #include "api_arrayelt.h"
 #include "context.h"
-#include "colormac.h"
 #include "light.h"
 #include "macros.h"
-#include "mtypes.h"
 #include "dlist.h"
 #include "dispatch.h"
 
 
-/* In states where certain vertex components are required for t&l or
- * rasterization, we still need to keep track of the current values.
- * These functions provide this service by keeping uptodate the
- * 'ctx->Current' struct for all data elements not included in the
- * currently enabled hardware vertex.
- * I.e. these functions would typically be used when outside of glBegin/End.
+/**
+ * \file
+ * Just update the ctx->Current vertex attributes.
+ * These functions are used when outside glBegin/glEnd or outside display
+ * lists.
  */
 
 
-void GLAPIENTRY _mesa_noop_EdgeFlag( GLboolean b )
+static void GLAPIENTRY _mesa_noop_EdgeFlag( GLboolean b )
 {
    GET_CURRENT_CONTEXT(ctx);
    ctx->Current.EdgeFlag = b;
 }
 
-void GLAPIENTRY _mesa_noop_EdgeFlagv( const GLboolean *b )
+static void GLAPIENTRY _mesa_noop_Indexf( GLfloat f )
 {
    GET_CURRENT_CONTEXT(ctx);
-   ctx->Current.EdgeFlag = *b;
+   ctx->Current.Attrib[VERT_ATTRIB_COLOR_INDEX][0] = f;
 }
 
-void GLAPIENTRY _mesa_noop_Indexf( GLfloat f )
+static void GLAPIENTRY _mesa_noop_Indexfv( const GLfloat *v )
 {
    GET_CURRENT_CONTEXT(ctx);
-   ctx->Current.Index = f;
+   ctx->Current.Attrib[VERT_ATTRIB_COLOR_INDEX][0] = *v;
 }
 
-void GLAPIENTRY _mesa_noop_Indexfv( const GLfloat *v )
-{
-   GET_CURRENT_CONTEXT(ctx);
-   ctx->Current.Index = *v;
-}
-
-void GLAPIENTRY _mesa_noop_FogCoordfEXT( GLfloat a )
+static void GLAPIENTRY _mesa_noop_FogCoordfEXT( GLfloat a )
 {
    GET_CURRENT_CONTEXT(ctx);
    GLfloat *dest = ctx->Current.Attrib[VERT_ATTRIB_FOG];
@@ -79,7 +70,7 @@ void GLAPIENTRY _mesa_noop_FogCoordfEXT( GLfloat a )
    dest[3] = 1.0;
 }
 
-void GLAPIENTRY _mesa_noop_FogCoordfvEXT( const GLfloat *v )
+static void GLAPIENTRY _mesa_noop_FogCoordfvEXT( const GLfloat *v )
 {
    GET_CURRENT_CONTEXT(ctx);
    GLfloat *dest = ctx->Current.Attrib[VERT_ATTRIB_FOG];
@@ -89,7 +80,7 @@ void GLAPIENTRY _mesa_noop_FogCoordfvEXT( const GLfloat *v )
    dest[3] = 1.0;
 }
 
-void GLAPIENTRY _mesa_noop_Normal3f( GLfloat a, GLfloat b, GLfloat c )
+static void GLAPIENTRY _mesa_noop_Normal3f( GLfloat a, GLfloat b, GLfloat c )
 {
    GET_CURRENT_CONTEXT(ctx);
    GLfloat *dest = ctx->Current.Attrib[VERT_ATTRIB_NORMAL];
@@ -99,7 +90,7 @@ void GLAPIENTRY _mesa_noop_Normal3f( GLfloat a, GLfloat b, GLfloat c )
    dest[3] = 1.0;
 }
 
-void GLAPIENTRY _mesa_noop_Normal3fv( const GLfloat *v )
+static void GLAPIENTRY _mesa_noop_Normal3fv( const GLfloat *v )
 {
    GET_CURRENT_CONTEXT(ctx);
    GLfloat *dest = ctx->Current.Attrib[VERT_ATTRIB_NORMAL];
@@ -109,7 +100,7 @@ void GLAPIENTRY _mesa_noop_Normal3fv( const GLfloat *v )
    dest[3] = 1.0;
 }
 
-void GLAPIENTRY _mesa_noop_Color4f( GLfloat a, GLfloat b, GLfloat c, GLfloat d )
+static void GLAPIENTRY _mesa_noop_Color4f( GLfloat a, GLfloat b, GLfloat c, GLfloat d )
 {
    GET_CURRENT_CONTEXT(ctx);
    GLfloat *color = ctx->Current.Attrib[VERT_ATTRIB_COLOR0];
@@ -119,7 +110,7 @@ void GLAPIENTRY _mesa_noop_Color4f( GLfloat a, GLfloat b, GLfloat c, GLfloat d )
    color[3] = d;
 }
 
-void GLAPIENTRY _mesa_noop_Color4fv( const GLfloat *v )
+static void GLAPIENTRY _mesa_noop_Color4fv( const GLfloat *v )
 {
    GET_CURRENT_CONTEXT(ctx);
    GLfloat *color = ctx->Current.Attrib[VERT_ATTRIB_COLOR0];
@@ -129,7 +120,7 @@ void GLAPIENTRY _mesa_noop_Color4fv( const GLfloat *v )
    color[3] = v[3];
 }
 
-void GLAPIENTRY _mesa_noop_Color3f( GLfloat a, GLfloat b, GLfloat c )
+static void GLAPIENTRY _mesa_noop_Color3f( GLfloat a, GLfloat b, GLfloat c )
 {
    GET_CURRENT_CONTEXT(ctx);
    GLfloat *color = ctx->Current.Attrib[VERT_ATTRIB_COLOR0];
@@ -139,7 +130,7 @@ void GLAPIENTRY _mesa_noop_Color3f( GLfloat a, GLfloat b, GLfloat c )
    color[3] = 1.0;
 }
 
-void GLAPIENTRY _mesa_noop_Color3fv( const GLfloat *v )
+static void GLAPIENTRY _mesa_noop_Color3fv( const GLfloat *v )
 {
    GET_CURRENT_CONTEXT(ctx);
    GLfloat *color = ctx->Current.Attrib[VERT_ATTRIB_COLOR0];
@@ -149,7 +140,7 @@ void GLAPIENTRY _mesa_noop_Color3fv( const GLfloat *v )
    color[3] = 1.0;
 }
 
-void GLAPIENTRY _mesa_noop_MultiTexCoord1fARB( GLenum target, GLfloat a )
+static void GLAPIENTRY _mesa_noop_MultiTexCoord1fARB( GLenum target, GLfloat a )
 {
    GET_CURRENT_CONTEXT(ctx);
    GLuint unit = target - GL_TEXTURE0_ARB;
@@ -166,7 +157,7 @@ void GLAPIENTRY _mesa_noop_MultiTexCoord1fARB( GLenum target, GLfloat a )
    }
 }
 
-void GLAPIENTRY _mesa_noop_MultiTexCoord1fvARB( GLenum target, const GLfloat *v )
+static void GLAPIENTRY _mesa_noop_MultiTexCoord1fvARB( GLenum target, const GLfloat *v )
 {
    GET_CURRENT_CONTEXT(ctx);
    GLuint unit = target - GL_TEXTURE0_ARB;
@@ -183,7 +174,7 @@ void GLAPIENTRY _mesa_noop_MultiTexCoord1fvARB( GLenum target, const GLfloat *v
    }
 }
 
-void GLAPIENTRY _mesa_noop_MultiTexCoord2fARB( GLenum target, GLfloat a, GLfloat b )
+static void GLAPIENTRY _mesa_noop_MultiTexCoord2fARB( GLenum target, GLfloat a, GLfloat b )
 {
    GET_CURRENT_CONTEXT(ctx);
    GLuint unit = target - GL_TEXTURE0_ARB;
@@ -200,7 +191,7 @@ void GLAPIENTRY _mesa_noop_MultiTexCoord2fARB( GLenum target, GLfloat a, GLfloat
    }
 }
 
-void GLAPIENTRY _mesa_noop_MultiTexCoord2fvARB( GLenum target, const GLfloat *v )
+static void GLAPIENTRY _mesa_noop_MultiTexCoord2fvARB( GLenum target, const GLfloat *v )
 {
    GET_CURRENT_CONTEXT(ctx);
    GLuint unit = target - GL_TEXTURE0_ARB;
@@ -217,7 +208,7 @@ void GLAPIENTRY _mesa_noop_MultiTexCoord2fvARB( GLenum target, const GLfloat *v
    }
 }
 
-void GLAPIENTRY _mesa_noop_MultiTexCoord3fARB( GLenum target, GLfloat a, GLfloat b, GLfloat c)
+static void GLAPIENTRY _mesa_noop_MultiTexCoord3fARB( GLenum target, GLfloat a, GLfloat b, GLfloat c)
 {
    GET_CURRENT_CONTEXT(ctx);
    GLuint unit = target - GL_TEXTURE0_ARB;
@@ -234,7 +225,7 @@ void GLAPIENTRY _mesa_noop_MultiTexCoord3fARB( GLenum target, GLfloat a, GLfloat
    }
 }
 
-void GLAPIENTRY _mesa_noop_MultiTexCoord3fvARB( GLenum target, const GLfloat *v )
+static void GLAPIENTRY _mesa_noop_MultiTexCoord3fvARB( GLenum target, const GLfloat *v )
 {
    GET_CURRENT_CONTEXT(ctx);
    GLuint unit = target - GL_TEXTURE0_ARB;
@@ -251,7 +242,7 @@ void GLAPIENTRY _mesa_noop_MultiTexCoord3fvARB( GLenum target, const GLfloat *v
    }
 }
 
-void GLAPIENTRY _mesa_noop_MultiTexCoord4fARB( GLenum target, GLfloat a, GLfloat b,
+static void GLAPIENTRY _mesa_noop_MultiTexCoord4fARB( GLenum target, GLfloat a, GLfloat b,
 			      GLfloat c, GLfloat d )
 {
    GET_CURRENT_CONTEXT(ctx);
@@ -269,7 +260,7 @@ void GLAPIENTRY _mesa_noop_MultiTexCoord4fARB( GLenum target, GLfloat a, GLfloat
    }
 }
 
-void GLAPIENTRY _mesa_noop_MultiTexCoord4fvARB( GLenum target, const GLfloat *v )
+static void GLAPIENTRY _mesa_noop_MultiTexCoord4fvARB( GLenum target, const GLfloat *v )
 {
    GET_CURRENT_CONTEXT(ctx);
    GLuint unit = target - GL_TEXTURE0_ARB;
@@ -286,7 +277,7 @@ void GLAPIENTRY _mesa_noop_MultiTexCoord4fvARB( GLenum target, const GLfloat *v
    }
 }
 
-void GLAPIENTRY _mesa_noop_SecondaryColor3fEXT( GLfloat a, GLfloat b, GLfloat c )
+static void GLAPIENTRY _mesa_noop_SecondaryColor3fEXT( GLfloat a, GLfloat b, GLfloat c )
 {
    GET_CURRENT_CONTEXT(ctx);
    GLfloat *color = ctx->Current.Attrib[VERT_ATTRIB_COLOR1];
@@ -296,7 +287,7 @@ void GLAPIENTRY _mesa_noop_SecondaryColor3fEXT( GLfloat a, GLfloat b, GLfloat c
    color[3] = 1.0;
 }
 
-void GLAPIENTRY _mesa_noop_SecondaryColor3fvEXT( const GLfloat *v )
+static void GLAPIENTRY _mesa_noop_SecondaryColor3fvEXT( const GLfloat *v )
 {
    GET_CURRENT_CONTEXT(ctx);
    GLfloat *color = ctx->Current.Attrib[VERT_ATTRIB_COLOR1];
@@ -306,7 +297,7 @@ void GLAPIENTRY _mesa_noop_SecondaryColor3fvEXT( const GLfloat *v )
    color[3] = 1.0;
 }
 
-void GLAPIENTRY _mesa_noop_TexCoord1f( GLfloat a )
+static void GLAPIENTRY _mesa_noop_TexCoord1f( GLfloat a )
 {
    GET_CURRENT_CONTEXT(ctx);
    GLfloat *dest = ctx->Current.Attrib[VERT_ATTRIB_TEX0];
@@ -316,7 +307,7 @@ void GLAPIENTRY _mesa_noop_TexCoord1f( GLfloat a )
    dest[3] = 1;
 }
 
-void GLAPIENTRY _mesa_noop_TexCoord1fv( const GLfloat *v )
+static void GLAPIENTRY _mesa_noop_TexCoord1fv( const GLfloat *v )
 {
    GET_CURRENT_CONTEXT(ctx);
    GLfloat *dest = ctx->Current.Attrib[VERT_ATTRIB_TEX0];
@@ -326,7 +317,7 @@ void GLAPIENTRY _mesa_noop_TexCoord1fv( const GLfloat *v )
    dest[3] = 1;
 }
 
-void GLAPIENTRY _mesa_noop_TexCoord2f( GLfloat a, GLfloat b )
+static void GLAPIENTRY _mesa_noop_TexCoord2f( GLfloat a, GLfloat b )
 {
    GET_CURRENT_CONTEXT(ctx);
    GLfloat *dest = ctx->Current.Attrib[VERT_ATTRIB_TEX0];
@@ -336,7 +327,7 @@ void GLAPIENTRY _mesa_noop_TexCoord2f( GLfloat a, GLfloat b )
    dest[3] = 1;
 }
 
-void GLAPIENTRY _mesa_noop_TexCoord2fv( const GLfloat *v )
+static void GLAPIENTRY _mesa_noop_TexCoord2fv( const GLfloat *v )
 {
    GET_CURRENT_CONTEXT(ctx);
    GLfloat *dest = ctx->Current.Attrib[VERT_ATTRIB_TEX0];
@@ -346,7 +337,7 @@ void GLAPIENTRY _mesa_noop_TexCoord2fv( const GLfloat *v )
    dest[3] = 1;
 }
 
-void GLAPIENTRY _mesa_noop_TexCoord3f( GLfloat a, GLfloat b, GLfloat c )
+static void GLAPIENTRY _mesa_noop_TexCoord3f( GLfloat a, GLfloat b, GLfloat c )
 {
    GET_CURRENT_CONTEXT(ctx);
    GLfloat *dest = ctx->Current.Attrib[VERT_ATTRIB_TEX0];
@@ -356,7 +347,7 @@ void GLAPIENTRY _mesa_noop_TexCoord3f( GLfloat a, GLfloat b, GLfloat c )
    dest[3] = 1;
 }
 
-void GLAPIENTRY _mesa_noop_TexCoord3fv( const GLfloat *v )
+static void GLAPIENTRY _mesa_noop_TexCoord3fv( const GLfloat *v )
 {
    GET_CURRENT_CONTEXT(ctx);
    GLfloat *dest = ctx->Current.Attrib[VERT_ATTRIB_TEX0];
@@ -366,7 +357,7 @@ void GLAPIENTRY _mesa_noop_TexCoord3fv( const GLfloat *v )
    dest[3] = 1;
 }
 
-void GLAPIENTRY _mesa_noop_TexCoord4f( GLfloat a, GLfloat b, GLfloat c, GLfloat d )
+static void GLAPIENTRY _mesa_noop_TexCoord4f( GLfloat a, GLfloat b, GLfloat c, GLfloat d )
 {
    GET_CURRENT_CONTEXT(ctx);
    GLfloat *dest = ctx->Current.Attrib[VERT_ATTRIB_TEX0];
@@ -376,7 +367,7 @@ void GLAPIENTRY _mesa_noop_TexCoord4f( GLfloat a, GLfloat b, GLfloat c, GLfloat
    dest[3] = d;
 }
 
-void GLAPIENTRY _mesa_noop_TexCoord4fv( const GLfloat *v )
+static void GLAPIENTRY _mesa_noop_TexCoord4fv( const GLfloat *v )
 {
    GET_CURRENT_CONTEXT(ctx);
    GLfloat *dest = ctx->Current.Attrib[VERT_ATTRIB_TEX0];
@@ -387,181 +378,189 @@ void GLAPIENTRY _mesa_noop_TexCoord4fv( const GLfloat *v )
 }
 
 
+/**
+ * GL_NV_vertex_program attributes.
+ * Note that these attributes alias the conventional vertex attributes.
+ */
 
-void GLAPIENTRY _mesa_noop_VertexAttrib1fNV( GLuint index, GLfloat x )
+static void GLAPIENTRY _mesa_noop_VertexAttrib1fNV( GLuint index, GLfloat x )
 {
    GET_CURRENT_CONTEXT(ctx);
-   if (index < VERT_ATTRIB_MAX) {
+   if (index < MAX_NV_VERTEX_PROGRAM_INPUTS) {
       ASSIGN_4V(ctx->Current.Attrib[index], x, 0, 0, 1);
    }
    else
-      _mesa_error( ctx, GL_INVALID_ENUM, "glVertexAttrib1fNV" );
+      _mesa_error( ctx, GL_INVALID_VALUE, "glVertexAttrib1fNV(index)" );
 }
 
-void GLAPIENTRY _mesa_noop_VertexAttrib1fvNV( GLuint index, const GLfloat *v )
+static void GLAPIENTRY _mesa_noop_VertexAttrib1fvNV( GLuint index, const GLfloat *v )
 {
    GET_CURRENT_CONTEXT(ctx);
-   if (index < VERT_ATTRIB_MAX) {
+   if (index < MAX_NV_VERTEX_PROGRAM_INPUTS) {
       ASSIGN_4V(ctx->Current.Attrib[index], v[0], 0, 0, 1);
    }
    else
-      _mesa_error( ctx, GL_INVALID_ENUM, "glVertexAttrib1fvNV" );
+      _mesa_error( ctx, GL_INVALID_VALUE, "glVertexAttrib1fvNV(index)" );
 }
 
-void GLAPIENTRY _mesa_noop_VertexAttrib2fNV( GLuint index, GLfloat x, GLfloat y )
+static void GLAPIENTRY _mesa_noop_VertexAttrib2fNV( GLuint index, GLfloat x, GLfloat y )
 {
    GET_CURRENT_CONTEXT(ctx);
-   if (index < VERT_ATTRIB_MAX) {
+   if (index < MAX_NV_VERTEX_PROGRAM_INPUTS) {
       ASSIGN_4V(ctx->Current.Attrib[index], x, y, 0, 1);
    }
    else
-      _mesa_error( ctx, GL_INVALID_ENUM, "glVertexAttrib2fNV" );
+      _mesa_error( ctx, GL_INVALID_VALUE, "glVertexAttrib2fNV(index)" );
 }
 
-void GLAPIENTRY _mesa_noop_VertexAttrib2fvNV( GLuint index, const GLfloat *v )
+static void GLAPIENTRY _mesa_noop_VertexAttrib2fvNV( GLuint index, const GLfloat *v )
 {
    GET_CURRENT_CONTEXT(ctx);
-   if (index < VERT_ATTRIB_MAX) {
+   if (index < MAX_NV_VERTEX_PROGRAM_INPUTS) {
       ASSIGN_4V(ctx->Current.Attrib[index], v[0], v[1], 0, 1);
    }
    else
-      _mesa_error( ctx, GL_INVALID_ENUM, "glVertexAttrib2fvNV" );
+      _mesa_error( ctx, GL_INVALID_VALUE, "glVertexAttrib2fvNV(index)" );
 }
 
-void GLAPIENTRY _mesa_noop_VertexAttrib3fNV( GLuint index, GLfloat x,
+static void GLAPIENTRY _mesa_noop_VertexAttrib3fNV( GLuint index, GLfloat x,
                                   GLfloat y, GLfloat z )
 {
    GET_CURRENT_CONTEXT(ctx);
-   if (index < VERT_ATTRIB_MAX) {
+   if (index < MAX_NV_VERTEX_PROGRAM_INPUTS) {
       ASSIGN_4V(ctx->Current.Attrib[index], x, y, z, 1);
    }
    else
-      _mesa_error( ctx, GL_INVALID_ENUM, "glVertexAttrib3fNV" );
+      _mesa_error( ctx, GL_INVALID_VALUE, "glVertexAttrib3fNV(index)" );
 }
 
-void GLAPIENTRY _mesa_noop_VertexAttrib3fvNV( GLuint index, const GLfloat *v )
+static void GLAPIENTRY _mesa_noop_VertexAttrib3fvNV( GLuint index, const GLfloat *v )
 {
    GET_CURRENT_CONTEXT(ctx);
-   if (index < VERT_ATTRIB_MAX) {
+   if (index < MAX_NV_VERTEX_PROGRAM_INPUTS) {
       ASSIGN_4V(ctx->Current.Attrib[index], v[0], v[1], v[2], 1);
    }
    else
-      _mesa_error( ctx, GL_INVALID_ENUM, "glVertexAttrib3fvNV" );
+      _mesa_error( ctx, GL_INVALID_VALUE, "glVertexAttrib3fvNV(index)" );
 }
 
-void GLAPIENTRY _mesa_noop_VertexAttrib4fNV( GLuint index, GLfloat x,
+static void GLAPIENTRY _mesa_noop_VertexAttrib4fNV( GLuint index, GLfloat x,
                                   GLfloat y, GLfloat z, GLfloat w )
 {
    GET_CURRENT_CONTEXT(ctx);
-   if (index < VERT_ATTRIB_MAX) {
+   if (index < MAX_NV_VERTEX_PROGRAM_INPUTS) {
       ASSIGN_4V(ctx->Current.Attrib[index], x, y, z, w);
    }
    else
-      _mesa_error( ctx, GL_INVALID_ENUM, "glVertexAttrib4fNV" );
+      _mesa_error( ctx, GL_INVALID_VALUE, "glVertexAttrib4fNV(index)" );
 }
 
-void GLAPIENTRY _mesa_noop_VertexAttrib4fvNV( GLuint index, const GLfloat *v )
+static void GLAPIENTRY _mesa_noop_VertexAttrib4fvNV( GLuint index, const GLfloat *v )
 {
    GET_CURRENT_CONTEXT(ctx);
-   if (index < VERT_ATTRIB_MAX) {
+   if (index < MAX_NV_VERTEX_PROGRAM_INPUTS) {
       ASSIGN_4V(ctx->Current.Attrib[index], v[0], v[1], v[2], v[3]);
    }
    else
-      _mesa_error( ctx, GL_INVALID_ENUM, "glVertexAttrib4fvNV" );
+      _mesa_error( ctx, GL_INVALID_VALUE, "glVertexAttrib4fvNV(index)" );
 }
 
 
-/*
- * XXX Un-alias attribs here
+
+/**
+ * GL_ARB_vertex_program attributes.
+ * Note that these attributes DO NOT alias the conventional vertex attributes.
  */
 
-void GLAPIENTRY _mesa_noop_VertexAttrib1fARB( GLuint index, GLfloat x )
+static void GLAPIENTRY _mesa_noop_VertexAttrib1fARB( GLuint index, GLfloat x )
 {
    GET_CURRENT_CONTEXT(ctx);
-   if (index < VERT_ATTRIB_MAX) {
-      ASSIGN_4V(ctx->Current.Attrib[index], x, 0, 0, 1);
+   if (index < MAX_VERTEX_ATTRIBS) {
+      ASSIGN_4V(ctx->Current.Attrib[VERT_ATTRIB_GENERIC0 + index], x, 0, 0, 1);
    }
    else
-      _mesa_error( ctx, GL_INVALID_ENUM, "glVertexAttrib1fARB" );
+      _mesa_error( ctx, GL_INVALID_VALUE, "glVertexAttrib1fARB(index)" );
 }
 
-void GLAPIENTRY _mesa_noop_VertexAttrib1fvARB( GLuint index, const GLfloat *v )
+static void GLAPIENTRY _mesa_noop_VertexAttrib1fvARB( GLuint index, const GLfloat *v )
 {
    GET_CURRENT_CONTEXT(ctx);
-   if (index < VERT_ATTRIB_MAX) {
-      ASSIGN_4V(ctx->Current.Attrib[index], v[0], 0, 0, 1);
+   if (index < MAX_VERTEX_ATTRIBS) {
+      ASSIGN_4V(ctx->Current.Attrib[VERT_ATTRIB_GENERIC0 + index], v[0], 0, 0, 1);
    }
    else
-      _mesa_error( ctx, GL_INVALID_ENUM, "glVertexAttrib1fvARB" );
+      _mesa_error( ctx, GL_INVALID_VALUE, "glVertexAttrib1fvARB(index)" );
 }
 
-void GLAPIENTRY _mesa_noop_VertexAttrib2fARB( GLuint index, GLfloat x, GLfloat y )
+static void GLAPIENTRY _mesa_noop_VertexAttrib2fARB( GLuint index, GLfloat x, GLfloat y )
 {
    GET_CURRENT_CONTEXT(ctx);
-   if (index < VERT_ATTRIB_MAX) {
-      ASSIGN_4V(ctx->Current.Attrib[index], x, y, 0, 1);
+   if (index < MAX_VERTEX_ATTRIBS) {
+      ASSIGN_4V(ctx->Current.Attrib[VERT_ATTRIB_GENERIC0 + index], x, y, 0, 1);
    }
    else
-      _mesa_error( ctx, GL_INVALID_ENUM, "glVertexAttrib2fARB" );
+      _mesa_error( ctx, GL_INVALID_VALUE, "glVertexAttrib2fARB(index)" );
 }
 
-void GLAPIENTRY _mesa_noop_VertexAttrib2fvARB( GLuint index, const GLfloat *v )
+static void GLAPIENTRY _mesa_noop_VertexAttrib2fvARB( GLuint index, const GLfloat *v )
 {
    GET_CURRENT_CONTEXT(ctx);
-   if (index < VERT_ATTRIB_MAX) {
-      ASSIGN_4V(ctx->Current.Attrib[index], v[0], v[1], 0, 1);
+   if (index < MAX_VERTEX_ATTRIBS) {
+      ASSIGN_4V(ctx->Current.Attrib[VERT_ATTRIB_GENERIC0 + index], v[0], v[1], 0, 1);
    }
    else
-      _mesa_error( ctx, GL_INVALID_ENUM, "glVertexAttrib2fvARB" );
+      _mesa_error( ctx, GL_INVALID_VALUE, "glVertexAttrib2fvARB(index)" );
 }
 
-void GLAPIENTRY _mesa_noop_VertexAttrib3fARB( GLuint index, GLfloat x,
+static void GLAPIENTRY _mesa_noop_VertexAttrib3fARB( GLuint index, GLfloat x,
                                   GLfloat y, GLfloat z )
 {
    GET_CURRENT_CONTEXT(ctx);
-   if (index < VERT_ATTRIB_MAX) {
-      ASSIGN_4V(ctx->Current.Attrib[index], x, y, z, 1);
+   if (index < MAX_VERTEX_ATTRIBS) {
+      ASSIGN_4V(ctx->Current.Attrib[VERT_ATTRIB_GENERIC0 + index], x, y, z, 1);
    }
    else
-      _mesa_error( ctx, GL_INVALID_ENUM, "glVertexAttrib3fARB" );
+      _mesa_error( ctx, GL_INVALID_VALUE, "glVertexAttrib3fARB(index)" );
 }
 
-void GLAPIENTRY _mesa_noop_VertexAttrib3fvARB( GLuint index, const GLfloat *v )
+static void GLAPIENTRY _mesa_noop_VertexAttrib3fvARB( GLuint index, const GLfloat *v )
 {
    GET_CURRENT_CONTEXT(ctx);
-   if (index < VERT_ATTRIB_MAX) {
-      ASSIGN_4V(ctx->Current.Attrib[index], v[0], v[1], v[2], 1);
+   if (index < MAX_VERTEX_ATTRIBS) {
+      ASSIGN_4V(ctx->Current.Attrib[VERT_ATTRIB_GENERIC0 + index], v[0], v[1], v[2], 1);
    }
    else
-      _mesa_error( ctx, GL_INVALID_ENUM, "glVertexAttrib3fvARB" );
+      _mesa_error( ctx, GL_INVALID_VALUE, "glVertexAttrib3fvARB(index)" );
 }
 
-void GLAPIENTRY _mesa_noop_VertexAttrib4fARB( GLuint index, GLfloat x,
+static void GLAPIENTRY _mesa_noop_VertexAttrib4fARB( GLuint index, GLfloat x,
                                   GLfloat y, GLfloat z, GLfloat w )
 {
    GET_CURRENT_CONTEXT(ctx);
-   if (index < VERT_ATTRIB_MAX) {
-      ASSIGN_4V(ctx->Current.Attrib[index], x, y, z, w);
+   if (index < MAX_VERTEX_ATTRIBS) {
+      ASSIGN_4V(ctx->Current.Attrib[VERT_ATTRIB_GENERIC0 + index], x, y, z, w);
    }
    else
-      _mesa_error( ctx, GL_INVALID_ENUM, "glVertexAttrib4fARB" );
+      _mesa_error( ctx, GL_INVALID_VALUE, "glVertexAttrib4fARB(index)" );
 }
 
-void GLAPIENTRY _mesa_noop_VertexAttrib4fvARB( GLuint index, const GLfloat *v )
+static void GLAPIENTRY _mesa_noop_VertexAttrib4fvARB( GLuint index, const GLfloat *v )
 {
    GET_CURRENT_CONTEXT(ctx);
-   if (index < VERT_ATTRIB_MAX) {
-      ASSIGN_4V(ctx->Current.Attrib[index], v[0], v[1], v[2], v[3]);
+   if (index < MAX_VERTEX_ATTRIBS) {
+      ASSIGN_4V(ctx->Current.Attrib[VERT_ATTRIB_GENERIC0 + index], v[0], v[1], v[2], v[3]);
    }
    else
-      _mesa_error( ctx, GL_INVALID_ENUM, "glVertexAttrib4fvARB" );
+      _mesa_error( ctx, GL_INVALID_VALUE, "glVertexAttrib4fvARB(index)" );
 }
 
 
 
-/* Material
+/**
+ * Called by glMaterial*()
  */
-void GLAPIENTRY _mesa_noop_Materialfv( GLenum face, GLenum pname, const GLfloat *params )
+void GLAPIENTRY
+_mesa_noop_Materialfv( GLenum face, GLenum pname, const GLfloat *params )
 {
    GET_CURRENT_CONTEXT(ctx);
    GLint i, nr;
@@ -575,7 +574,7 @@ void GLAPIENTRY _mesa_noop_Materialfv( GLenum face, GLenum pname, const GLfloat
    if (bitmask == 0)
       return;
 
-   switch (face) {
+   switch (pname) {
    case GL_SHININESS: nr = 1; break;
    case GL_COLOR_INDEXES: nr = 3; break;
    default: nr = 4 ; break;
@@ -588,66 +587,68 @@ void GLAPIENTRY _mesa_noop_Materialfv( GLenum face, GLenum pname, const GLfloat
    _mesa_update_material( ctx, bitmask );
 }
 
-/* These really are noops outside begin/end:
+
+/**
+ * These really are noops outside begin/end:
  */
-void GLAPIENTRY _mesa_noop_Vertex2fv( const GLfloat *v )
+static void GLAPIENTRY _mesa_noop_Vertex2fv( const GLfloat *v )
 {
    (void) v;
 }
 
-void GLAPIENTRY _mesa_noop_Vertex3fv( const GLfloat *v )
+static void GLAPIENTRY _mesa_noop_Vertex3fv( const GLfloat *v )
 {
    (void) v;
 }
 
-void GLAPIENTRY _mesa_noop_Vertex4fv( const GLfloat *v )
+static void GLAPIENTRY _mesa_noop_Vertex4fv( const GLfloat *v )
 {
    (void) v;
 }
 
-void GLAPIENTRY _mesa_noop_Vertex2f( GLfloat a, GLfloat b )
+static void GLAPIENTRY _mesa_noop_Vertex2f( GLfloat a, GLfloat b )
 {
    (void) a; (void) b;
 }
 
-void GLAPIENTRY _mesa_noop_Vertex3f( GLfloat a, GLfloat b, GLfloat c )
+static void GLAPIENTRY _mesa_noop_Vertex3f( GLfloat a, GLfloat b, GLfloat c )
 {
    (void) a; (void) b; (void) c;
 }
 
-void GLAPIENTRY _mesa_noop_Vertex4f( GLfloat a, GLfloat b, GLfloat c, GLfloat d )
+static void GLAPIENTRY _mesa_noop_Vertex4f( GLfloat a, GLfloat b, GLfloat c, GLfloat d )
 {
    (void) a; (void) b; (void) c; (void) d;
 }
 
 /* Similarly, these have no effect outside begin/end:
  */
-void GLAPIENTRY _mesa_noop_EvalCoord1f( GLfloat a )
+static void GLAPIENTRY _mesa_noop_EvalCoord1f( GLfloat a )
 {
    (void) a;
 }
 
-void GLAPIENTRY _mesa_noop_EvalCoord1fv( const GLfloat *v )
+static void GLAPIENTRY _mesa_noop_EvalCoord1fv( const GLfloat *v )
 {
    (void) v;
 }
 
-void GLAPIENTRY _mesa_noop_EvalCoord2f( GLfloat a, GLfloat b )
+static void GLAPIENTRY _mesa_noop_EvalCoord2f( GLfloat a, GLfloat b )
 {
    (void) a; (void) b;
 }
 
-void GLAPIENTRY _mesa_noop_EvalCoord2fv( const GLfloat *v )
+static void GLAPIENTRY _mesa_noop_EvalCoord2fv( const GLfloat *v )
 {
    (void) v;
 }
 
-void GLAPIENTRY _mesa_noop_EvalPoint1( GLint a )
+static void GLAPIENTRY _mesa_noop_EvalPoint1( GLint a )
 {
    (void) a;
 }
 
-void GLAPIENTRY _mesa_noop_EvalPoint2( GLint a, GLint b )
+static void GLAPIENTRY _mesa_noop_EvalPoint2( GLint a, GLint b )
 {
    (void) a; (void) b;
 }
@@ -656,7 +657,7 @@ void GLAPIENTRY _mesa_noop_EvalPoint2( GLint a, GLint b )
 /* Begin -- call into driver, should result in the vtxfmt being
  * swapped out:
  */
-void GLAPIENTRY _mesa_noop_Begin( GLenum mode )
+static void GLAPIENTRY _mesa_noop_Begin( GLenum mode )
 {
    (void) mode;
 }
@@ -664,14 +665,15 @@ void GLAPIENTRY _mesa_noop_Begin( GLenum mode )
 
 /* End -- just raise an error
  */
-void GLAPIENTRY _mesa_noop_End( void )
+static void GLAPIENTRY _mesa_noop_End( void )
 {
    GET_CURRENT_CONTEXT(ctx);
-   _mesa_error( ctx, GL_INVALID_OPERATION, "glEnd" );
+   _mesa_error( ctx, GL_INVALID_OPERATION, "glEnd(no glBegin)" );
 }
 
 
-/* Execute a glRectf() function.  This is not suitable for GL_COMPILE
+/**
+ * Execute a glRectf() function.  This is not suitable for GL_COMPILE
  * modes (as the test for outside begin/end is not compiled),
  * but may be useful for drivers in circumstances which exclude
  * display list interactions.
@@ -679,7 +681,8 @@ void GLAPIENTRY _mesa_noop_End( void )
  * (None of the functions in this file are suitable for GL_COMPILE
  * modes).
  */
-void GLAPIENTRY _mesa_noop_Rectf( GLfloat x1, GLfloat y1, GLfloat x2, GLfloat y2 )
+void GLAPIENTRY
+_mesa_noop_Rectf( GLfloat x1, GLfloat y1, GLfloat x2, GLfloat y2 )
 {
    {
       GET_CURRENT_CONTEXT(ctx);
@@ -695,11 +698,13 @@ void GLAPIENTRY _mesa_noop_Rectf( GLfloat x1, GLfloat y1, GLfloat x2, GLfloat y2
 }
 
 
-/* Some very basic support for arrays.  Drivers without explicit array
+/**
+ * Some very basic support for arrays.  Drivers without explicit array
  * support can hook these in, but still need to supply an array-elt
  * implementation.
  */
-void GLAPIENTRY _mesa_noop_DrawArrays(GLenum mode, GLint start, GLsizei count)
+static void GLAPIENTRY
+_mesa_noop_DrawArrays(GLenum mode, GLint start, GLsizei count)
 {
    GET_CURRENT_CONTEXT(ctx);
    GLint i;
@@ -714,8 +719,9 @@ void GLAPIENTRY _mesa_noop_DrawArrays(GLenum mode, GLint start, GLsizei count)
 }
 
 
-void GLAPIENTRY _mesa_noop_DrawElements(GLenum mode, GLsizei count, GLenum type,
-			     const GLvoid *indices)
+static void GLAPIENTRY
+_mesa_noop_DrawElements(GLenum mode, GLsizei count, GLenum type,
+                        const GLvoid *indices)
 {
    GET_CURRENT_CONTEXT(ctx);
    GLint i;
@@ -746,10 +752,12 @@ void GLAPIENTRY _mesa_noop_DrawElements(GLenum mode, GLsizei count, GLenum type,
    CALL_End(GET_DISPATCH(), ());
 }
 
-void GLAPIENTRY _mesa_noop_DrawRangeElements(GLenum mode,
-				  GLuint start, GLuint end,
-				  GLsizei count, GLenum type,
-				  const GLvoid *indices)
+
+static void GLAPIENTRY
+_mesa_noop_DrawRangeElements(GLenum mode,
+                             GLuint start, GLuint end,
+                             GLsizei count, GLenum type,
+                             const GLvoid *indices)
 {
    GET_CURRENT_CONTEXT(ctx);
 
@@ -763,16 +771,18 @@ void GLAPIENTRY _mesa_noop_DrawRangeElements(GLenum mode,
  * Eval Mesh
  */
 
-/* KW: If are compiling, we don't know whether eval will produce a
- *     vertex when it is run in the future.  If this is pure immediate
- *     mode, eval is a noop if neither vertex map is enabled.
+/**
+ * KW:
+ * If are compiling, we don't know whether eval will produce a
+ * vertex when it is run in the future.  If this is pure immediate
+ * mode, eval is a noop if neither vertex map is enabled.
  *
- *     Thus we need to have a check in the display list code or
- *     elsewhere for eval(1,2) vertices in the case where
- *     map(1,2)_vertex is disabled, and to purge those vertices from
- *     the vb.
+ * Thus we need to have a check in the display list code or elsewhere
+ * for eval(1,2) vertices in the case where map(1,2)_vertex is
+ * disabled, and to purge those vertices from the vb.
  */
-void GLAPIENTRY _mesa_noop_EvalMesh1( GLenum mode, GLint i1, GLint i2 )
+void GLAPIENTRY
+_mesa_noop_EvalMesh1( GLenum mode, GLint i1, GLint i2 )
 {
    GET_CURRENT_CONTEXT(ctx);
    GLint i;
@@ -810,7 +820,8 @@ void GLAPIENTRY _mesa_noop_EvalMesh1( GLenum mode, GLint i1, GLint i2 )
 
 
 
-void GLAPIENTRY _mesa_noop_EvalMesh2( GLenum mode, GLint i1, GLint i2, GLint j1, GLint j2 )
+void GLAPIENTRY
+_mesa_noop_EvalMesh2( GLenum mode, GLint i1, GLint i2, GLint j1, GLint j2 )
 {
    GET_CURRENT_CONTEXT(ctx);
    GLfloat u, du, v, dv, v1, u1;
@@ -882,12 +893,14 @@ void GLAPIENTRY _mesa_noop_EvalMesh2( GLenum mode, GLint i1, GLint i2, GLint j1,
 
 
 
-/* Build a vertexformat full of things to use outside begin/end pairs.
+/**
+ * Build a vertexformat of functions to use outside begin/end pairs.
  * 
  * TODO -- build a whole dispatch table for this purpose, and likewise
  * for inside begin/end.
  */
-void _mesa_noop_vtxfmt_init( GLvertexformat *vfmt )
+void
+_mesa_noop_vtxfmt_init( GLvertexformat *vfmt )
 {
    vfmt->ArrayElement = _ae_loopback_array_elt;	        /* generic helper */
    vfmt->Begin = _mesa_noop_Begin;
@@ -898,7 +911,6 @@ void _mesa_noop_vtxfmt_init( GLvertexformat *vfmt )
    vfmt->Color4f = _mesa_noop_Color4f;
    vfmt->Color4fv = _mesa_noop_Color4fv;
    vfmt->EdgeFlag = _mesa_noop_EdgeFlag;
-   vfmt->EdgeFlagv = _mesa_noop_EdgeFlagv;
    vfmt->End = _mesa_noop_End;
    vfmt->EvalCoord1f = _mesa_noop_EvalCoord1f;
    vfmt->EvalCoord1fv = _mesa_noop_EvalCoord1fv;
