@@ -1,8 +1,8 @@
 /*
  * Mesa 3-D graphics library
- * Version:  6.5
+ * Version:  6.5.2
  *
- * Copyright (C) 1999-2005  Brian Paul   All Rights Reserved.
+ * Copyright (C) 1999-2006  Brian Paul   All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -42,28 +42,23 @@
 #include "texformat.h"
 #include "texstore.h"
 
-#if USE_EXTERNAL_DXTN_LIB
-#ifdef __MINGW32__
-/* no dlopen */
-#define DXTN_EXT "dxtn.dll"
-#define DXTN_PREFIX ""
-#define dlopen(name, mode) LoadLibrary(name)
-#define dlsym(hndl, proc) GetProcAddress(hndl, proc)
-#define dlclose(hndl) FreeLibrary(hndl)
-#elif defined(__DJGPP__)
-/* has dlopen, but doesn't like the names */
+#if USE_EXTERNAL_DXTN_LIB && !defined(__MINGW32__)
 #include <dlfcn.h>
-#define DXTN_EXT "dxtn.dxe"
-#define DXTN_PREFIX "_"
-#else
-/* happiness */
-#include <dlfcn.h>
-#define DXTN_EXT "libtxc_dxtn.so"
-#define DXTN_PREFIX ""
 #endif
-#endif /* USE_EXTERNAL_DXTN_LIB */
+
+#ifdef __MINGW32__
+#define DXTN_LIBNAME "dxtn.dll"
+#define RTLD_LAZY 0
+#define RTLD_GLOBAL 0
+#elif defined(__DJGPP__)
+#define DXTN_LIBNAME "dxtn.dxe"
+#else
+#define DXTN_LIBNAME "libtxc_dxtn.so"
+#endif
+
 
 typedef void (*dxtFetchTexelFuncExt)( GLint srcRowstride, GLubyte *pixdata, GLint col, GLint row, GLvoid *texelOut );
+
 dxtFetchTexelFuncExt fetch_ext_rgb_dxt1 = NULL;
 dxtFetchTexelFuncExt fetch_ext_rgba_dxt1 = NULL;
 dxtFetchTexelFuncExt fetch_ext_rgba_dxt3 = NULL;
@@ -73,11 +68,78 @@ typedef void (*dxtCompressTexFuncExt)(GLint srccomps, GLint width,
                                       GLint height, const GLchan *srcPixData,
                                       GLenum destformat, GLubyte *dest,
                                       GLint dstRowStride);
+
 static dxtCompressTexFuncExt ext_tx_compress_dxtn = NULL;
 
 static void *dxtlibhandle = NULL;
 
 
+typedef void (*GenericFunc)(void);
+
+
+/**
+ * Wrapper for dlopen().
+ * XXX Probably move this and the following wrappers into imports.h someday.
+ */
+static void *
+_mesa_dlopen(const char *libname, int flags)
+{
+#if USE_EXTERNAL_DXTN_LIB
+#ifdef __MINGW32__
+   return LoadLibrary(libname);
+#else
+   return dlopen(libname, flags);
+#endif
+#else
+   return (GenericFunc) NULL;
+#endif /* USE_EXTERNAL_DXTN_LIB */
+}
+
+
+/**
+ * Wrapper for dlsym() that does a cast to a generic function type,
+ * rather than a void *.  This reduces the number of warnings that are
+ * generated.
+ */
+static GenericFunc
+_mesa_dlsym(void *handle, const char *fname)
+{
+#if USE_EXTERNAL_DXTN_LIB
+#ifdef __MINGW32__
+   return (GenericFunc) GetProcAddress(handle, fname);
+#elif defined(__DJGPP__)
+   /* need '_' prefix on symbol names */
+   char fname2[1000];
+   fname2[0] = '_';
+   _mesa_strncpy(fname2 + 1, fname, 998);
+   fname2[999] = 0;
+   return (GenericFunc) dlsym(handle, fname2);
+#else
+   return (GenericFunc) dlsym(handle, fname);
+#endif
+#else
+   return (GenericFunc) NULL;
+#endif /* USE_EXTERNAL_DXTN_LIB */
+}
+
+
+/**
+ * Wrapper for dlclose().
+ */
+static void
+_mesa_dlclose(void *handle)
+{
+#if USE_EXTERNAL_DXTN_LIB
+#ifdef __MINGW32__
+   FreeLibrary(handle);
+#else
+   dlclose(handle);
+#endif
+#endif
+}
+
+
+
 void
 _mesa_init_texture_s3tc( GLcontext *ctx )
 {
@@ -85,44 +147,45 @@ _mesa_init_texture_s3tc( GLcontext *ctx )
    ctx->Mesa_DXTn = GL_FALSE;
 #if USE_EXTERNAL_DXTN_LIB
    if (!dxtlibhandle) {
-      dxtlibhandle = dlopen (DXTN_EXT, RTLD_LAZY | RTLD_GLOBAL);
+      dxtlibhandle = _mesa_dlopen(DXTN_LIBNAME, RTLD_LAZY | RTLD_GLOBAL);
       if (!dxtlibhandle) {
-	 _mesa_warning(ctx, "couldn't open " DXTN_EXT ", software DXTn "
-	    "compression/decompression unavailable\n");
+	 _mesa_warning(ctx, "couldn't open " DXTN_LIBNAME ", software DXTn "
+	    "compression/decompression unavailable");
       }
       else {
          /* the fetch functions are not per context! Might be problematic... */
-         fetch_ext_rgb_dxt1 = (dxtFetchTexelFuncExt)dlsym(dxtlibhandle, DXTN_PREFIX "fetch_2d_texel_rgb_dxt1");
-         if (fetch_ext_rgb_dxt1 != NULL) {
-            fetch_ext_rgba_dxt1 = (dxtFetchTexelFuncExt)dlsym(dxtlibhandle, DXTN_PREFIX "fetch_2d_texel_rgba_dxt1");
-         }
-         if (fetch_ext_rgba_dxt1 != NULL) {
-            fetch_ext_rgba_dxt3 = (dxtFetchTexelFuncExt)dlsym(dxtlibhandle, DXTN_PREFIX "fetch_2d_texel_rgba_dxt3");
-         }
-         if (fetch_ext_rgba_dxt3 != NULL) {
-            fetch_ext_rgba_dxt5 = (dxtFetchTexelFuncExt)dlsym(dxtlibhandle, DXTN_PREFIX "fetch_2d_texel_rgba_dxt5");
-         }
-         if (fetch_ext_rgba_dxt5 != NULL) {
-            ext_tx_compress_dxtn = (dxtCompressTexFuncExt)dlsym(dxtlibhandle, DXTN_PREFIX "tx_compress_dxtn");
-         }
+         fetch_ext_rgb_dxt1 = (dxtFetchTexelFuncExt)
+            _mesa_dlsym(dxtlibhandle, "fetch_2d_texel_rgb_dxt1");
+         fetch_ext_rgba_dxt1 = (dxtFetchTexelFuncExt)
+            _mesa_dlsym(dxtlibhandle, "fetch_2d_texel_rgba_dxt1");
+         fetch_ext_rgba_dxt3 = (dxtFetchTexelFuncExt)
+            _mesa_dlsym(dxtlibhandle, "fetch_2d_texel_rgba_dxt3");
+         fetch_ext_rgba_dxt5 = (dxtFetchTexelFuncExt)
+            _mesa_dlsym(dxtlibhandle, "fetch_2d_texel_rgba_dxt5");
+         ext_tx_compress_dxtn = (dxtCompressTexFuncExt)
+            _mesa_dlsym(dxtlibhandle, "tx_compress_dxtn");
 
-         if (ext_tx_compress_dxtn == NULL) {
+         if (!fetch_ext_rgb_dxt1 ||
+             !fetch_ext_rgba_dxt1 ||
+             !fetch_ext_rgba_dxt3 ||
+             !fetch_ext_rgba_dxt5 ||
+             !ext_tx_compress_dxtn) {
 	    _mesa_warning(ctx, "couldn't reference all symbols in "
-	       DXTN_EXT ", software DXTn compression/decompression "
-	       "unavailable\n");
+	       DXTN_LIBNAME ", software DXTn compression/decompression "
+	       "unavailable");
             fetch_ext_rgb_dxt1 = NULL;
             fetch_ext_rgba_dxt1 = NULL;
             fetch_ext_rgba_dxt3 = NULL;
             fetch_ext_rgba_dxt5 = NULL;
             ext_tx_compress_dxtn = NULL;
-            dlclose(dxtlibhandle);
+            _mesa_dlclose(dxtlibhandle);
             dxtlibhandle = NULL;
          }
       }
    }
    if (dxtlibhandle) {
       ctx->Mesa_DXTn = GL_TRUE;
-      _mesa_warning(ctx, "software DXTn compression/decompression available\n");
+      _mesa_warning(ctx, "software DXTn compression/decompression available");
    }
 #else
    (void) ctx;
@@ -133,7 +196,7 @@ _mesa_init_texture_s3tc( GLcontext *ctx )
  * Called via TexFormat->StoreImage to store an RGB_DXT1 texture.
  */
 static GLboolean
-texstore_rgb_dxt1(STORE_PARAMS)
+texstore_rgb_dxt1(TEXSTORE_PARAMS)
 {
    const GLchan *pixels;
    GLint srcRowStride;
@@ -145,7 +208,8 @@ texstore_rgb_dxt1(STORE_PARAMS)
    ASSERT(dstXoffset % 4 == 0);
    ASSERT(dstYoffset % 4 == 0);
    ASSERT(dstZoffset % 4 == 0);
-   (void) dstZoffset; (void) dstImageStride;
+   (void) dstZoffset;
+   (void) dstImageOffsets;
 
    if (srcFormat != GL_RGB ||
        srcType != CHAN_TYPE ||
@@ -172,11 +236,13 @@ texstore_rgb_dxt1(STORE_PARAMS)
    }
 
    dst = _mesa_compressed_image_address(dstXoffset, dstYoffset, 0,
-                                        GL_COMPRESSED_RGB_S3TC_DXT1_EXT,
+                                        dstFormat->MesaFormat,
                                         texWidth, (GLubyte *) dstAddr);
 
    if (ext_tx_compress_dxtn) {
-      (*ext_tx_compress_dxtn)(3, srcWidth, srcHeight, pixels,  GL_COMPRESSED_RGB_S3TC_DXT1_EXT, dst, dstRowStride);
+      (*ext_tx_compress_dxtn)(3, srcWidth, srcHeight, pixels,
+                              GL_COMPRESSED_RGB_S3TC_DXT1_EXT,
+                              dst, dstRowStride);
    }
    else {
       _mesa_problem(ctx, "external dxt library not available");
@@ -193,7 +259,7 @@ texstore_rgb_dxt1(STORE_PARAMS)
  * Called via TexFormat->StoreImage to store an RGBA_DXT1 texture.
  */
 static GLboolean
-texstore_rgba_dxt1(STORE_PARAMS)
+texstore_rgba_dxt1(TEXSTORE_PARAMS)
 {
    const GLchan *pixels;
    GLint srcRowStride;
@@ -205,7 +271,8 @@ texstore_rgba_dxt1(STORE_PARAMS)
    ASSERT(dstXoffset % 4 == 0);
    ASSERT(dstYoffset % 4 == 0);
    ASSERT(dstZoffset % 4 == 0);
-   (void) dstZoffset; (void) dstImageStride;
+   (void) dstZoffset;
+   (void) dstImageOffsets;
 
    if (srcFormat != GL_RGBA ||
        srcType != CHAN_TYPE ||
@@ -232,10 +299,12 @@ texstore_rgba_dxt1(STORE_PARAMS)
    }
 
    dst = _mesa_compressed_image_address(dstXoffset, dstYoffset, 0,
-                                        GL_COMPRESSED_RGBA_S3TC_DXT1_EXT,
+                                        dstFormat->MesaFormat,
                                         texWidth, (GLubyte *) dstAddr);
    if (ext_tx_compress_dxtn) {
-      (*ext_tx_compress_dxtn)(4, srcWidth, srcHeight, pixels,  GL_COMPRESSED_RGBA_S3TC_DXT1_EXT, dst, dstRowStride);
+      (*ext_tx_compress_dxtn)(4, srcWidth, srcHeight, pixels,
+                              GL_COMPRESSED_RGBA_S3TC_DXT1_EXT,
+                              dst, dstRowStride);
    }
    else {
       _mesa_problem(ctx, "external dxt library not available");
@@ -252,7 +321,7 @@ texstore_rgba_dxt1(STORE_PARAMS)
  * Called via TexFormat->StoreImage to store an RGBA_DXT3 texture.
  */
 static GLboolean
-texstore_rgba_dxt3(STORE_PARAMS)
+texstore_rgba_dxt3(TEXSTORE_PARAMS)
 {
    const GLchan *pixels;
    GLint srcRowStride;
@@ -264,7 +333,8 @@ texstore_rgba_dxt3(STORE_PARAMS)
    ASSERT(dstXoffset % 4 == 0);
    ASSERT(dstYoffset % 4 == 0);
    ASSERT(dstZoffset % 4 == 0);
-   (void) dstZoffset; (void) dstImageStride;
+   (void) dstZoffset;
+   (void) dstImageOffsets;
 
    if (srcFormat != GL_RGBA ||
        srcType != CHAN_TYPE ||
@@ -290,10 +360,12 @@ texstore_rgba_dxt3(STORE_PARAMS)
    }
 
    dst = _mesa_compressed_image_address(dstXoffset, dstYoffset, 0,
-                                        GL_COMPRESSED_RGBA_S3TC_DXT3_EXT,
+                                        dstFormat->MesaFormat,
                                         texWidth, (GLubyte *) dstAddr);
    if (ext_tx_compress_dxtn) {
-      (*ext_tx_compress_dxtn)(4, srcWidth, srcHeight, pixels,  GL_COMPRESSED_RGBA_S3TC_DXT3_EXT, dst, dstRowStride);
+      (*ext_tx_compress_dxtn)(4, srcWidth, srcHeight, pixels,
+                              GL_COMPRESSED_RGBA_S3TC_DXT3_EXT,
+                              dst, dstRowStride);
    }
    else {
       _mesa_problem(ctx, "external dxt library not available");
@@ -310,7 +382,7 @@ texstore_rgba_dxt3(STORE_PARAMS)
  * Called via TexFormat->StoreImage to store an RGBA_DXT5 texture.
  */
 static GLboolean
-texstore_rgba_dxt5(STORE_PARAMS)
+texstore_rgba_dxt5(TEXSTORE_PARAMS)
 {
    const GLchan *pixels;
    GLint srcRowStride;
@@ -322,7 +394,8 @@ texstore_rgba_dxt5(STORE_PARAMS)
    ASSERT(dstXoffset % 4 == 0);
    ASSERT(dstYoffset % 4 == 0);
    ASSERT(dstZoffset % 4 == 0);
-   (void) dstZoffset; (void) dstImageStride;
+   (void) dstZoffset;
+   (void) dstImageOffsets;
 
    if (srcFormat != GL_RGBA ||
        srcType != CHAN_TYPE ||
@@ -348,10 +421,12 @@ texstore_rgba_dxt5(STORE_PARAMS)
    }
 
    dst = _mesa_compressed_image_address(dstXoffset, dstYoffset, 0,
-                                        GL_COMPRESSED_RGBA_S3TC_DXT5_EXT,
+                                        dstFormat->MesaFormat,
                                         texWidth, (GLubyte *) dstAddr);
    if (ext_tx_compress_dxtn) {
-      (*ext_tx_compress_dxtn)(4, srcWidth, srcHeight, pixels,  GL_COMPRESSED_RGBA_S3TC_DXT5_EXT, dst, dstRowStride);
+      (*ext_tx_compress_dxtn)(4, srcWidth, srcHeight, pixels,
+                              GL_COMPRESSED_RGBA_S3TC_DXT5_EXT,
+                              dst, dstRowStride);
    }
    else {
       _mesa_problem(ctx, "external dxt library not available");
@@ -368,12 +443,14 @@ static void
 fetch_texel_2d_rgb_dxt1( const struct gl_texture_image *texImage,
                          GLint i, GLint j, GLint k, GLchan *texel )
 {
-    (void) k;
-    if (fetch_ext_rgb_dxt1) {
-       ASSERT (sizeof(GLchan) == sizeof(GLubyte));
-       (*fetch_ext_rgb_dxt1)((texImage)->RowStride, (GLubyte *)(texImage)->Data, i, j, texel);
-    }
-    else _mesa_debug(NULL, "attempted to decode s3tc texture without library available\n");
+   (void) k;
+   if (fetch_ext_rgb_dxt1) {
+      ASSERT (sizeof(GLchan) == sizeof(GLubyte));
+      fetch_ext_rgb_dxt1(texImage->RowStride,
+                         (GLubyte *)(texImage)->Data, i, j, texel);
+   }
+   else
+      _mesa_debug(NULL, "attempted to decode s3tc texture without library available\n");
 }
 
 
@@ -397,9 +474,11 @@ fetch_texel_2d_rgba_dxt1( const struct gl_texture_image *texImage,
 {
    (void) k;
    if (fetch_ext_rgba_dxt1) {
-       (*fetch_ext_rgba_dxt1)((texImage)->RowStride, (GLubyte *)(texImage)->Data, i, j, texel);
-    }
-    else _mesa_debug(NULL, "attempted to decode s3tc texture without library available\n");
+      fetch_ext_rgba_dxt1(texImage->RowStride,
+                          (GLubyte *)(texImage)->Data, i, j, texel);
+   }
+   else
+      _mesa_debug(NULL, "attempted to decode s3tc texture without library available\n");
 }
 
 
@@ -423,10 +502,12 @@ fetch_texel_2d_rgba_dxt3( const struct gl_texture_image *texImage,
 {
    (void) k;
    if (fetch_ext_rgba_dxt3) {
-       ASSERT (sizeof(GLchan) == sizeof(GLubyte));
-       (*fetch_ext_rgba_dxt3)((texImage)->RowStride, (GLubyte *)(texImage)->Data, i, j, texel);
-    }
-    else _mesa_debug(NULL, "attempted to decode s3tc texture without library available\n");
+      ASSERT (sizeof(GLchan) == sizeof(GLubyte));
+      fetch_ext_rgba_dxt3(texImage->RowStride, (GLubyte *)(texImage)->Data,
+                          i, j, texel);
+   }
+   else
+      _mesa_debug(NULL, "attempted to decode s3tc texture without library available\n");
 }
 
 
@@ -450,9 +531,11 @@ fetch_texel_2d_rgba_dxt5( const struct gl_texture_image *texImage,
 {
    (void) k;
    if (fetch_ext_rgba_dxt5) {
-       (*fetch_ext_rgba_dxt5)((texImage)->RowStride, (GLubyte *)(texImage)->Data, i, j, texel);
-    }
-    else _mesa_debug(NULL, "attempted to decode s3tc texture without library available\n");
+      fetch_ext_rgba_dxt5(texImage->RowStride, (GLubyte *)(texImage)->Data,
+                          i, j, texel);
+   }
+   else
+      _mesa_debug(NULL, "attempted to decode s3tc texture without library available\n");
 }
 
 
@@ -491,6 +574,7 @@ const struct gl_texture_format _mesa_texformat_rgb_dxt1 = {
    NULL, /*impossible*/ 		/* FetchTexel1Df */
    fetch_texel_2d_f_rgb_dxt1, 		/* FetchTexel2Df */
    NULL, /*impossible*/ 		/* FetchTexel3Df */
+   NULL					/* StoreTexel */
 };
 
 const struct gl_texture_format _mesa_texformat_rgba_dxt1 = {
@@ -514,6 +598,7 @@ const struct gl_texture_format _mesa_texformat_rgba_dxt1 = {
    NULL, /*impossible*/ 		/* FetchTexel1Df */
    fetch_texel_2d_f_rgba_dxt1, 		/* FetchTexel2Df */
    NULL, /*impossible*/ 		/* FetchTexel3Df */
+   NULL					/* StoreTexel */
 };
 
 const struct gl_texture_format _mesa_texformat_rgba_dxt3 = {
@@ -537,6 +622,7 @@ const struct gl_texture_format _mesa_texformat_rgba_dxt3 = {
    NULL, /*impossible*/ 		/* FetchTexel1Df */
    fetch_texel_2d_f_rgba_dxt3, 		/* FetchTexel2Df */
    NULL, /*impossible*/ 		/* FetchTexel3Df */
+   NULL					/* StoreTexel */
 };
 
 const struct gl_texture_format _mesa_texformat_rgba_dxt5 = {
@@ -560,4 +646,5 @@ const struct gl_texture_format _mesa_texformat_rgba_dxt5 = {
    NULL, /*impossible*/ 		/* FetchTexel1Df */
    fetch_texel_2d_f_rgba_dxt5, 		/* FetchTexel2Df */
    NULL, /*impossible*/ 		/* FetchTexel3Df */
+   NULL					/* StoreTexel */
 };
